"""Generated client library for tpu version v2alpha1."""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.tpu.v2alpha1 import tpu_v2alpha1_messages as messages


class TpuV2alpha1(base_api.BaseApiClient):
  """Generated client library for service tpu version v2alpha1."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://tpu.googleapis.com/'
  MTLS_BASE_URL = 'https://tpu.mtls.googleapis.com/'

  _PACKAGE = 'tpu'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v2alpha1'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'TpuV2alpha1'
  _URL_VERSION = 'v2alpha1'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new tpu handle."""
    url = url or self.BASE_URL
    super(TpuV2alpha1, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_acceleratorTypes = self.ProjectsLocationsAcceleratorTypesService(self)
    self.projects_locations_nodes = self.ProjectsLocationsNodesService(self)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations_queuedResources = self.ProjectsLocationsQueuedResourcesService(self)
    self.projects_locations_reservations = self.ProjectsLocationsReservationsService(self)
    self.projects_locations_runtimeVersions = self.ProjectsLocationsRuntimeVersionsService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsAcceleratorTypesService(base_api.BaseApiService):
    """Service class for the projects_locations_acceleratorTypes resource."""

    _NAME = 'projects_locations_acceleratorTypes'

    def __init__(self, client):
      super(TpuV2alpha1.ProjectsLocationsAcceleratorTypesService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets AcceleratorType.

      Args:
        request: (TpuProjectsLocationsAcceleratorTypesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (AcceleratorType) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/acceleratorTypes/{acceleratorTypesId}',
        http_method='GET',
        method_id='tpu.projects.locations.acceleratorTypes.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2alpha1/{+name}',
        request_field='',
        request_type_name='TpuProjectsLocationsAcceleratorTypesGetRequest',
        response_type_name='AcceleratorType',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists accelerator types supported by this API.

      Args:
        request: (TpuProjectsLocationsAcceleratorTypesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListAcceleratorTypesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/acceleratorTypes',
        http_method='GET',
        method_id='tpu.projects.locations.acceleratorTypes.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v2alpha1/{+parent}/acceleratorTypes',
        request_field='',
        request_type_name='TpuProjectsLocationsAcceleratorTypesListRequest',
        response_type_name='ListAcceleratorTypesResponse',
        supports_download=False,
    )

  class ProjectsLocationsNodesService(base_api.BaseApiService):
    """Service class for the projects_locations_nodes resource."""

    _NAME = 'projects_locations_nodes'

    def __init__(self, client):
      super(TpuV2alpha1.ProjectsLocationsNodesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a node.

      Args:
        request: (TpuProjectsLocationsNodesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/nodes',
        http_method='POST',
        method_id='tpu.projects.locations.nodes.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['nodeId', 'requestId'],
        relative_path='v2alpha1/{+parent}/nodes',
        request_field='node',
        request_type_name='TpuProjectsLocationsNodesCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a node.

      Args:
        request: (TpuProjectsLocationsNodesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/nodes/{nodesId}',
        http_method='DELETE',
        method_id='tpu.projects.locations.nodes.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['requestId'],
        relative_path='v2alpha1/{+name}',
        request_field='',
        request_type_name='TpuProjectsLocationsNodesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the details of a node.

      Args:
        request: (TpuProjectsLocationsNodesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Node) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/nodes/{nodesId}',
        http_method='GET',
        method_id='tpu.projects.locations.nodes.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2alpha1/{+name}',
        request_field='',
        request_type_name='TpuProjectsLocationsNodesGetRequest',
        response_type_name='Node',
        supports_download=False,
    )

    def GetGuestAttributes(self, request, global_params=None):
      r"""Retrieves the guest attributes for the node.

      Args:
        request: (TpuProjectsLocationsNodesGetGuestAttributesRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GetGuestAttributesResponse) The response message.
      """
      config = self.GetMethodConfig('GetGuestAttributes')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetGuestAttributes.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/nodes/{nodesId}:getGuestAttributes',
        http_method='POST',
        method_id='tpu.projects.locations.nodes.getGuestAttributes',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2alpha1/{+name}:getGuestAttributes',
        request_field='getGuestAttributesRequest',
        request_type_name='TpuProjectsLocationsNodesGetGuestAttributesRequest',
        response_type_name='GetGuestAttributesResponse',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists nodes.

      Args:
        request: (TpuProjectsLocationsNodesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListNodesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/nodes',
        http_method='GET',
        method_id='tpu.projects.locations.nodes.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v2alpha1/{+parent}/nodes',
        request_field='',
        request_type_name='TpuProjectsLocationsNodesListRequest',
        response_type_name='ListNodesResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Updates the configurations of a node.

      Args:
        request: (TpuProjectsLocationsNodesPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/nodes/{nodesId}',
        http_method='PATCH',
        method_id='tpu.projects.locations.nodes.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v2alpha1/{+name}',
        request_field='node',
        request_type_name='TpuProjectsLocationsNodesPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def PerformMaintenance(self, request, global_params=None):
      r"""Perform manual maintenance on a node.

      Args:
        request: (TpuProjectsLocationsNodesPerformMaintenanceRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('PerformMaintenance')
      return self._RunMethod(
          config, request, global_params=global_params)

    PerformMaintenance.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/nodes/{nodesId}:performMaintenance',
        http_method='POST',
        method_id='tpu.projects.locations.nodes.performMaintenance',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2alpha1/{+name}:performMaintenance',
        request_field='performMaintenanceRequest',
        request_type_name='TpuProjectsLocationsNodesPerformMaintenanceRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def SimulateMaintenanceEvent(self, request, global_params=None):
      r"""Simulates a maintenance event.

      Args:
        request: (TpuProjectsLocationsNodesSimulateMaintenanceEventRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('SimulateMaintenanceEvent')
      return self._RunMethod(
          config, request, global_params=global_params)

    SimulateMaintenanceEvent.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/nodes/{nodesId}:simulateMaintenanceEvent',
        http_method='POST',
        method_id='tpu.projects.locations.nodes.simulateMaintenanceEvent',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2alpha1/{+name}:simulateMaintenanceEvent',
        request_field='simulateMaintenanceEventRequest',
        request_type_name='TpuProjectsLocationsNodesSimulateMaintenanceEventRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Start(self, request, global_params=None):
      r"""Starts a node.

      Args:
        request: (TpuProjectsLocationsNodesStartRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Start')
      return self._RunMethod(
          config, request, global_params=global_params)

    Start.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/nodes/{nodesId}:start',
        http_method='POST',
        method_id='tpu.projects.locations.nodes.start',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2alpha1/{+name}:start',
        request_field='startNodeRequest',
        request_type_name='TpuProjectsLocationsNodesStartRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Stop(self, request, global_params=None):
      r"""Stops a node. This operation is only available with single TPU nodes.

      Args:
        request: (TpuProjectsLocationsNodesStopRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Stop')
      return self._RunMethod(
          config, request, global_params=global_params)

    Stop.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/nodes/{nodesId}:stop',
        http_method='POST',
        method_id='tpu.projects.locations.nodes.stop',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2alpha1/{+name}:stop',
        request_field='stopNodeRequest',
        request_type_name='TpuProjectsLocationsNodesStopRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(TpuV2alpha1.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Starts asynchronous cancellation on a long-running operation. The server makes a best effort to cancel the operation, but success is not guaranteed. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`. Clients can use Operations.GetOperation or other methods to check whether the cancellation succeeded or whether the operation completed despite cancellation. On successful cancellation, the operation is not deleted; instead, it becomes an operation with an Operation.error value with a google.rpc.Status.code of `1`, corresponding to `Code.CANCELLED`.

      Args:
        request: (TpuProjectsLocationsOperationsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}:cancel',
        http_method='POST',
        method_id='tpu.projects.locations.operations.cancel',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2alpha1/{+name}:cancel',
        request_field='',
        request_type_name='TpuProjectsLocationsOperationsCancelRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a long-running operation. This method indicates that the client is no longer interested in the operation result. It does not cancel the operation. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`.

      Args:
        request: (TpuProjectsLocationsOperationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='DELETE',
        method_id='tpu.projects.locations.operations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2alpha1/{+name}',
        request_field='',
        request_type_name='TpuProjectsLocationsOperationsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (TpuProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='tpu.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2alpha1/{+name}',
        request_field='',
        request_type_name='TpuProjectsLocationsOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`.

      Args:
        request: (TpuProjectsLocationsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/operations',
        http_method='GET',
        method_id='tpu.projects.locations.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken', 'returnPartialSuccess'],
        relative_path='v2alpha1/{+name}/operations',
        request_field='',
        request_type_name='TpuProjectsLocationsOperationsListRequest',
        response_type_name='ListOperationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsQueuedResourcesService(base_api.BaseApiService):
    """Service class for the projects_locations_queuedResources resource."""

    _NAME = 'projects_locations_queuedResources'

    def __init__(self, client):
      super(TpuV2alpha1.ProjectsLocationsQueuedResourcesService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a QueuedResource TPU instance.

      Args:
        request: (TpuProjectsLocationsQueuedResourcesCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/queuedResources',
        http_method='POST',
        method_id='tpu.projects.locations.queuedResources.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['queuedResourceId', 'requestId'],
        relative_path='v2alpha1/{+parent}/queuedResources',
        request_field='queuedResource',
        request_type_name='TpuProjectsLocationsQueuedResourcesCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a QueuedResource TPU instance.

      Args:
        request: (TpuProjectsLocationsQueuedResourcesDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/queuedResources/{queuedResourcesId}',
        http_method='DELETE',
        method_id='tpu.projects.locations.queuedResources.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['force', 'requestId'],
        relative_path='v2alpha1/{+name}',
        request_field='',
        request_type_name='TpuProjectsLocationsQueuedResourcesDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a queued resource.

      Args:
        request: (TpuProjectsLocationsQueuedResourcesGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (QueuedResource) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/queuedResources/{queuedResourcesId}',
        http_method='GET',
        method_id='tpu.projects.locations.queuedResources.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2alpha1/{+name}',
        request_field='',
        request_type_name='TpuProjectsLocationsQueuedResourcesGetRequest',
        response_type_name='QueuedResource',
        supports_download=False,
    )

    def GetMaintenanceInfo(self, request, global_params=None):
      r"""Gets the maintenance info for a queued resource.

      Args:
        request: (TpuProjectsLocationsQueuedResourcesGetMaintenanceInfoRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GetMaintenanceInfoResponse) The response message.
      """
      config = self.GetMethodConfig('GetMaintenanceInfo')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetMaintenanceInfo.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/queuedResources/{queuedResourcesId}:getMaintenanceInfo',
        http_method='GET',
        method_id='tpu.projects.locations.queuedResources.getMaintenanceInfo',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2alpha1/{+name}:getMaintenanceInfo',
        request_field='',
        request_type_name='TpuProjectsLocationsQueuedResourcesGetMaintenanceInfoRequest',
        response_type_name='GetMaintenanceInfoResponse',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists queued resources.

      Args:
        request: (TpuProjectsLocationsQueuedResourcesListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListQueuedResourcesResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/queuedResources',
        http_method='GET',
        method_id='tpu.projects.locations.queuedResources.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v2alpha1/{+parent}/queuedResources',
        request_field='',
        request_type_name='TpuProjectsLocationsQueuedResourcesListRequest',
        response_type_name='ListQueuedResourcesResponse',
        supports_download=False,
    )

    def PerformMaintenanceQueuedResource(self, request, global_params=None):
      r"""Perform manual maintenance on specific nodes of a QueuedResource.

      Args:
        request: (TpuProjectsLocationsQueuedResourcesPerformMaintenanceQueuedResourceRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('PerformMaintenanceQueuedResource')
      return self._RunMethod(
          config, request, global_params=global_params)

    PerformMaintenanceQueuedResource.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/queuedResources/{queuedResourcesId}:performMaintenanceQueuedResource',
        http_method='POST',
        method_id='tpu.projects.locations.queuedResources.performMaintenanceQueuedResource',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2alpha1/{+name}:performMaintenanceQueuedResource',
        request_field='performMaintenanceQueuedResourceRequest',
        request_type_name='TpuProjectsLocationsQueuedResourcesPerformMaintenanceQueuedResourceRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Reset(self, request, global_params=None):
      r"""Resets a QueuedResource TPU instance.

      Args:
        request: (TpuProjectsLocationsQueuedResourcesResetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Reset')
      return self._RunMethod(
          config, request, global_params=global_params)

    Reset.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/queuedResources/{queuedResourcesId}:reset',
        http_method='POST',
        method_id='tpu.projects.locations.queuedResources.reset',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2alpha1/{+name}:reset',
        request_field='resetQueuedResourceRequest',
        request_type_name='TpuProjectsLocationsQueuedResourcesResetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsReservationsService(base_api.BaseApiService):
    """Service class for the projects_locations_reservations resource."""

    _NAME = 'projects_locations_reservations'

    def __init__(self, client):
      super(TpuV2alpha1.ProjectsLocationsReservationsService, self).__init__(client)
      self._upload_configs = {
          }

    def List(self, request, global_params=None):
      r"""Retrieves the reservations for the given project in the given location.

      Args:
        request: (TpuProjectsLocationsReservationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListReservationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/reservations',
        http_method='GET',
        method_id='tpu.projects.locations.reservations.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['pageSize', 'pageToken'],
        relative_path='v2alpha1/{+parent}/reservations',
        request_field='',
        request_type_name='TpuProjectsLocationsReservationsListRequest',
        response_type_name='ListReservationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsRuntimeVersionsService(base_api.BaseApiService):
    """Service class for the projects_locations_runtimeVersions resource."""

    _NAME = 'projects_locations_runtimeVersions'

    def __init__(self, client):
      super(TpuV2alpha1.ProjectsLocationsRuntimeVersionsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets a runtime version.

      Args:
        request: (TpuProjectsLocationsRuntimeVersionsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (RuntimeVersion) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/runtimeVersions/{runtimeVersionsId}',
        http_method='GET',
        method_id='tpu.projects.locations.runtimeVersions.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2alpha1/{+name}',
        request_field='',
        request_type_name='TpuProjectsLocationsRuntimeVersionsGetRequest',
        response_type_name='RuntimeVersion',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists runtime versions supported by this API.

      Args:
        request: (TpuProjectsLocationsRuntimeVersionsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListRuntimeVersionsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}/runtimeVersions',
        http_method='GET',
        method_id='tpu.projects.locations.runtimeVersions.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v2alpha1/{+parent}/runtimeVersions',
        request_field='',
        request_type_name='TpuProjectsLocationsRuntimeVersionsListRequest',
        response_type_name='ListRuntimeVersionsResponse',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(TpuV2alpha1.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def GenerateServiceIdentity(self, request, global_params=None):
      r"""Generates the Cloud TPU service identity for the project.

      Args:
        request: (TpuProjectsLocationsGenerateServiceIdentityRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GenerateServiceIdentityResponse) The response message.
      """
      config = self.GetMethodConfig('GenerateServiceIdentity')
      return self._RunMethod(
          config, request, global_params=global_params)

    GenerateServiceIdentity.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}:generateServiceIdentity',
        http_method='POST',
        method_id='tpu.projects.locations.generateServiceIdentity',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=[],
        relative_path='v2alpha1/{+parent}:generateServiceIdentity',
        request_field='generateServiceIdentityRequest',
        request_type_name='TpuProjectsLocationsGenerateServiceIdentityRequest',
        response_type_name='GenerateServiceIdentityResponse',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (TpuProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='tpu.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v2alpha1/{+name}',
        request_field='',
        request_type_name='TpuProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service. This method lists locations based on the resource scope provided in the ListLocationsRequest.name field: * **Global locations**: If `name` is empty, the method lists the public locations available to all projects. * **Project-specific locations**: If `name` follows the format `projects/{project}`, the method lists locations visible to that specific project. This includes public, private, or other project-specific locations enabled for the project. For gRPC and client library implementations, the resource name is passed as the `name` field. For direct service calls, the resource name is incorporated into the request path based on the specific service implementation and version.

      Args:
        request: (TpuProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v2alpha1/projects/{projectsId}/locations',
        http_method='GET',
        method_id='tpu.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['extraLocationTypes', 'filter', 'pageSize', 'pageToken'],
        relative_path='v2alpha1/{+name}/locations',
        request_field='',
        request_type_name='TpuProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(TpuV2alpha1.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



"""Generated message classes for looker version v1.

"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'looker'


class AdminSettings(_messages.Message):
  r"""Looker instance Admin settings fields.

  Fields:
    allowedEmailDomains: Email domain allowlist for the instance.
  """

  allowedEmailDomains = _messages.StringField(1, repeated=True)


class CancelOperationRequest(_messages.Message):
  r"""The request message for Operations.CancelOperation."""


class ControlledEgressConfig(_messages.Message):
  r"""Controlled egress configuration.

  Fields:
    egressFqdns: Optional. List of fully qualified domain names to be added to
      the allowlist for outbound traffic.
    marketplaceEnabled: Optional. Whether marketplace is enabled.
    webProxyIps: Output only. The list of IP addresses used by Secure Web
      Proxy for outbound traffic.
  """

  egressFqdns = _messages.StringField(1, repeated=True)
  marketplaceEnabled = _messages.BooleanField(2)
  webProxyIps = _messages.StringField(3, repeated=True)


class CustomDomain(_messages.Message):
  r"""Custom domain information.

  Enums:
    StateValueValuesEnum: Domain state.

  Fields:
    domain: Domain name.
    state: Domain state.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Domain state.

    Values:
      CUSTOM_DOMAIN_STATE_UNSPECIFIED: Unspecified state.
      UNVERIFIED: DNS record is not created.
      VERIFIED: DNS record is created.
      MODIFYING: Calling SLM to update.
      AVAILABLE: ManagedCertificate is ready.
      UNAVAILABLE: ManagedCertificate is not ready.
      UNKNOWN: Status is not known.
    """
    CUSTOM_DOMAIN_STATE_UNSPECIFIED = 0
    UNVERIFIED = 1
    VERIFIED = 2
    MODIFYING = 3
    AVAILABLE = 4
    UNAVAILABLE = 5
    UNKNOWN = 6

  domain = _messages.StringField(1)
  state = _messages.EnumField('StateValueValuesEnum', 2)


class Date(_messages.Message):
  r"""Represents a whole or partial calendar date, such as a birthday. The
  time of day and time zone are either specified elsewhere or are
  insignificant. The date is relative to the Gregorian Calendar. This can
  represent one of the following: * A full date, with non-zero year, month,
  and day values. * A month and day, with a zero year (for example, an
  anniversary). * A year on its own, with a zero month and a zero day. * A
  year and month, with a zero day (for example, a credit card expiration
  date). Related types: * google.type.TimeOfDay * google.type.DateTime *
  google.protobuf.Timestamp

  Fields:
    day: Day of a month. Must be from 1 to 31 and valid for the year and
      month, or 0 to specify a year by itself or a year and month where the
      day isn't significant.
    month: Month of a year. Must be from 1 to 12, or 0 to specify a year
      without a month and day.
    year: Year of the date. Must be from 1 to 9999, or 0 to specify a date
      without a year.
  """

  day = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  month = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  year = _messages.IntegerField(3, variant=_messages.Variant.INT32)


class DenyMaintenancePeriod(_messages.Message):
  r"""Specifies the maintenance denial period.

  Fields:
    endDate: Required. End date of the deny maintenance period.
    startDate: Required. Start date of the deny maintenance period.
    time: Required. Time in UTC when the period starts and ends.
  """

  endDate = _messages.MessageField('Date', 1)
  startDate = _messages.MessageField('Date', 2)
  time = _messages.MessageField('TimeOfDay', 3)


class Empty(_messages.Message):
  r"""A generic empty message that you can re-use to avoid defining duplicated
  empty messages in your APIs. A typical example is to use it as the request
  or the response type of an API method. For instance: service Foo { rpc
  Bar(google.protobuf.Empty) returns (google.protobuf.Empty); }
  """



class EncryptionConfig(_messages.Message):
  r"""Encryption configuration (i.e. CMEK).

  Enums:
    KmsKeyStateValueValuesEnum: Output only. Status of the CMEK key.

  Fields:
    kmsKeyName: Name of the CMEK key in KMS (input parameter).
    kmsKeyNameVersion: Output only. Full name and version of the CMEK key
      currently in use to encrypt Looker data. Format: `projects/{project}/loc
      ations/{location}/keyRings/{ring}/cryptoKeys/{key}/cryptoKeyVersions/{ve
      rsion}`. Empty if CMEK is not configured in this instance.
    kmsKeyState: Output only. Status of the CMEK key.
  """

  class KmsKeyStateValueValuesEnum(_messages.Enum):
    r"""Output only. Status of the CMEK key.

    Values:
      KMS_KEY_STATE_UNSPECIFIED: CMEK status not specified.
      VALID: CMEK key is currently valid.
      REVOKED: CMEK key is currently revoked (instance should in restricted
        mode).
    """
    KMS_KEY_STATE_UNSPECIFIED = 0
    VALID = 1
    REVOKED = 2

  kmsKeyName = _messages.StringField(1)
  kmsKeyNameVersion = _messages.StringField(2)
  kmsKeyState = _messages.EnumField('KmsKeyStateValueValuesEnum', 3)


class ExportEncryptionConfig(_messages.Message):
  r"""Configuration for Encryption - e.g. CMEK.

  Fields:
    kmsKeyName: Required. Name of the CMEK key in KMS.
  """

  kmsKeyName = _messages.StringField(1)


class ExportInstanceRequest(_messages.Message):
  r"""Request options for exporting data of an Instance.

  Fields:
    encryptionConfig: Required. Encryption configuration (CMEK). For CMEK
      enabled instances it should be same as looker CMEK.
    gcsUri: The path to the folder in Google Cloud Storage where the export
      will be stored. The URI is in the form `gs://bucketName/folderName`.
  """

  encryptionConfig = _messages.MessageField('ExportEncryptionConfig', 1)
  gcsUri = _messages.StringField(2)


class ExportMetadata(_messages.Message):
  r"""ExportMetadata represents the metadata of the exported artifacts. The
  metadata.json file in export artifact can be parsed as this message

  Enums:
    SourceValueValuesEnum: The source type of the migration.

  Fields:
    exportEncryptionKey: Encryption key that was used to encrypt the export
      artifacts.
    filePaths: List of files created as part of export artifact (excluding the
      metadata). The paths are relative to the folder containing the metadata.
    lookerEncryptionKey: Looker encryption key, encrypted with the provided
      export encryption key. This value will only be populated if the looker
      instance uses Looker managed encryption instead of CMEK.
    lookerInstance: Name of the exported instance. Format:
      projects/{project}/locations/{location}/instances/{instance}
    lookerPlatformEdition: Platform edition of the exported instance.
    lookerVersion: Version of instance when the export was created.
    source: The source type of the migration.
  """

  class SourceValueValuesEnum(_messages.Enum):
    r"""The source type of the migration.

    Values:
      SOURCE_UNSPECIFIED: Source not specified
      LOOKER_CORE: Source of export is Looker Core
      LOOKER_ORIGINAL: Source of export is Looker Original
    """
    SOURCE_UNSPECIFIED = 0
    LOOKER_CORE = 1
    LOOKER_ORIGINAL = 2

  exportEncryptionKey = _messages.MessageField('ExportMetadataEncryptionKey', 1)
  filePaths = _messages.StringField(2, repeated=True)
  lookerEncryptionKey = _messages.StringField(3)
  lookerInstance = _messages.StringField(4)
  lookerPlatformEdition = _messages.StringField(5)
  lookerVersion = _messages.StringField(6)
  source = _messages.EnumField('SourceValueValuesEnum', 7)


class ExportMetadataEncryptionKey(_messages.Message):
  r"""Encryption key details for the exported artifact.

  Fields:
    cmek: Name of the CMEK.
    version: Version of the CMEK.
  """

  cmek = _messages.StringField(1)
  version = _messages.StringField(2)


class ImportInstanceRequest(_messages.Message):
  r"""Requestion options for importing looker data to an Instance

  Fields:
    gcsUri: Path to the import folder in Google Cloud Storage, in the form
      `gs://bucketName/folderName`.
  """

  gcsUri = _messages.StringField(1)


class IngressIpAllowlistConfig(_messages.Message):
  r"""Ingress IP allowlist configuration.

  Fields:
    allowlistRules: Optional. List of IP range rules to allow ingress traffic.
    enabled: Optional. Whether ingress IP allowlist functionality is enabled
      on the Looker instance.
    googleServicesEnabled: Optional. Whether google service connections are
      enabled for the instance.
  """

  allowlistRules = _messages.MessageField('IngressIpAllowlistRule', 1, repeated=True)
  enabled = _messages.BooleanField(2)
  googleServicesEnabled = _messages.BooleanField(3)


class IngressIpAllowlistRule(_messages.Message):
  r"""Ingress IP allowlist rule.

  Fields:
    description: Optional. Description for the IP range.
    ipRange: Optional. The IP range to allow ingress traffic from.
  """

  description = _messages.StringField(1)
  ipRange = _messages.StringField(2)


class Instance(_messages.Message):
  r"""A Looker instance.

  Enums:
    ClassTypeValueValuesEnum: Optional. Storage class of the instance.
    PlatformEditionValueValuesEnum: Platform edition.
    ReleaseChannelValueValuesEnum: Optional. The selected release channel for
      the instance.
    SoftDeleteReasonValueValuesEnum: Output only. The reason for the instance
      being in a soft-deleted state.
    StateValueValuesEnum: Output only. The state of the instance.

  Fields:
    acceleratedSecurityPatchEnabled: Optional. Accelerated security patch
      enabled for the instance.
    adminSettings: Looker Instance Admin settings.
    catalogIntegrationOptOut: Optional. Indicates whether catalog integration
      is disabled for the Looker instance.
    classType: Optional. Storage class of the instance.
    consumerNetwork: Network name in the consumer project. Format:
      `projects/{project}/global/networks/{network}`. Note that the consumer
      network may be in a different GCP project than the consumer project that
      is hosting the Looker Instance.
    controlledEgressConfig: Optional. Controlled egress configuration.
    controlledEgressEnabled: Optional. Whether controlled egress is enabled on
      the Looker instance.
    createTime: Output only. The time when the Looker instance provisioning
      was first requested.
    customDomain: Custom domain configuration for the instance.
    denyMaintenancePeriod: Maintenance denial period for this instance.
    egressPublicIp: Output only. Public Egress IP (IPv4).
    encryptionConfig: Encryption configuration (CMEK). Only set if CMEK has
      been enabled on the instance.
    fipsEnabled: Optional. Whether FIPS is enabled on the Looker instance.
    geminiEnabled: Optional. Whether Gemini feature is enabled on the Looker
      instance or not.
    ingressIpAllowlistConfig: Optional. Ingress IP allowlist configuration for
      the Looker instance.
    ingressPrivateIp: Output only. Private Ingress IP (IPv4).
    ingressPublicIp: Output only. Public Ingress IP (IPv4).
    lastDenyMaintenancePeriod: Output only. Last computed maintenance denial
      period for this instance.
    linkedLspProjectNumber: Optional. Linked Google Cloud Project Number for
      Looker Studio Pro.
    lookerUri: Output only. Looker instance URI which can be used to access
      the Looker Instance UI.
    lookerVersion: Output only. The Looker version that the instance is using.
    maintenanceSchedule: Maintenance schedule for this instance.
    maintenanceWindow: Maintenance window for this instance.
    name: Output only. Format:
      `projects/{project}/locations/{location}/instances/{instance}`.
    oauthConfig: Looker instance OAuth login settings.
    periodicExportConfig: Optional. Configuration for periodic export.
    platformEdition: Platform edition.
    privateIpEnabled: Whether private IP is enabled on the Looker instance.
    pscConfig: Optional. PSC configuration. Used when `psc_enabled` is true.
    pscEnabled: Optional. Whether to use Private Service Connect (PSC) for
      private IP connectivity. If true, neither `public_ip_enabled` nor
      `private_ip_enabled` can be true.
    publicIpEnabled: Whether public IP is enabled on the Looker instance.
    releaseChannel: Optional. The selected release channel for the instance.
    reservedRange: Name of a reserved IP address range within the
      Instance.consumer_network, to be used for private services access
      connection. May or may not be specified in a create request.
    satisfiesPzi: Output only. Reserved for future use.
    satisfiesPzs: Output only. Reserved for future use.
    softDeleteReason: Output only. The reason for the instance being in a
      soft-deleted state.
    state: Output only. The state of the instance.
    suspendedTime: Output only. The time when the Looker instance was
      suspended (soft deleted).
    updateTime: Output only. The time when the Looker instance was last
      updated.
    userMetadata: Optional. User metadata.
  """

  class ClassTypeValueValuesEnum(_messages.Enum):
    r"""Optional. Storage class of the instance.

    Values:
      CLASS_TYPE_UNSPECIFIED: Unspecified storage class.
      R1: Filestore.
      P1: PD SSD.
    """
    CLASS_TYPE_UNSPECIFIED = 0
    R1 = 1
    P1 = 2

  class PlatformEditionValueValuesEnum(_messages.Enum):
    r"""Platform edition.

    Values:
      PLATFORM_EDITION_UNSPECIFIED: Represents an unspecified platform
        edition.
      LOOKER_CORE_TRIAL: Represents the Looker Core Trial edition.
      LOOKER_CORE_STANDARD: Represents the Looker Core Standard edition.
      LOOKER_CORE_STANDARD_ANNUAL: Represents the Looker Core Standard Annual
        edition.
      LOOKER_CORE_ENTERPRISE_ANNUAL: Represents the Looker Core Enterprise
        Annual edition.
      LOOKER_CORE_EMBED_ANNUAL: Represents the Looker Core Embed Annual
        edition.
      LOOKER_CORE_NONPROD_STANDARD_ANNUAL: Represents the Looker Core Nonprod
        Standard Annual edition.
      LOOKER_CORE_NONPROD_ENTERPRISE_ANNUAL: Represents the Looker Core
        Nonprod Enterprise Annual edition.
      LOOKER_CORE_NONPROD_EMBED_ANNUAL: Represents the Looker Core Nonprod
        Embed Annual edition.
      LOOKER_CORE_TRIAL_STANDARD: Represents the Looker Core Trial Standard
        edition.
      LOOKER_CORE_TRIAL_ENTERPRISE: Represents the Looker Core Trial
        Enterprise edition.
      LOOKER_CORE_TRIAL_EMBED: Represents the Looker Core Trial Embed edition.
    """
    PLATFORM_EDITION_UNSPECIFIED = 0
    LOOKER_CORE_TRIAL = 1
    LOOKER_CORE_STANDARD = 2
    LOOKER_CORE_STANDARD_ANNUAL = 3
    LOOKER_CORE_ENTERPRISE_ANNUAL = 4
    LOOKER_CORE_EMBED_ANNUAL = 5
    LOOKER_CORE_NONPROD_STANDARD_ANNUAL = 6
    LOOKER_CORE_NONPROD_ENTERPRISE_ANNUAL = 7
    LOOKER_CORE_NONPROD_EMBED_ANNUAL = 8
    LOOKER_CORE_TRIAL_STANDARD = 9
    LOOKER_CORE_TRIAL_ENTERPRISE = 10
    LOOKER_CORE_TRIAL_EMBED = 11

  class ReleaseChannelValueValuesEnum(_messages.Enum):
    r"""Optional. The selected release channel for the instance.

    Values:
      RELEASE_CHANNEL_UNSPECIFIED: Unspecified release channel.
      RAPID: Rapid: Most frequent updates.
      REGULAR: Regular: Balanced, default for production.
      STABLE: Stable: Least frequent, for maximum stability.
    """
    RELEASE_CHANNEL_UNSPECIFIED = 0
    RAPID = 1
    REGULAR = 2
    STABLE = 3

  class SoftDeleteReasonValueValuesEnum(_messages.Enum):
    r"""Output only. The reason for the instance being in a soft-deleted
    state.

    Values:
      SOFT_DELETE_REASON_UNSPECIFIED: Soft delete reason is unspecified. This
        is the default value.
      BILLING_ACCOUNT_ISSUE: Instance is soft deleted due to billing account
        issues.
      TRIAL_EXPIRED: Instance is soft deleted due to trial expiration.
      CUSTOMER_REQUEST: Instance is soft deleted by the customer.
    """
    SOFT_DELETE_REASON_UNSPECIFIED = 0
    BILLING_ACCOUNT_ISSUE = 1
    TRIAL_EXPIRED = 2
    CUSTOMER_REQUEST = 3

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The state of the instance.

    Values:
      STATE_UNSPECIFIED: State is unspecified.
      ACTIVE: Instance is active and ready for use.
      CREATING: Instance provisioning is in progress.
      FAILED: Instance is in a failed state.
      SUSPENDED: Instance was suspended.
      UPDATING: Instance update is in progress.
      DELETING: Instance delete is in progress.
      EXPORTING: Instance is being exported.
      IMPORTING: Instance is importing data.
    """
    STATE_UNSPECIFIED = 0
    ACTIVE = 1
    CREATING = 2
    FAILED = 3
    SUSPENDED = 4
    UPDATING = 5
    DELETING = 6
    EXPORTING = 7
    IMPORTING = 8

  acceleratedSecurityPatchEnabled = _messages.BooleanField(1)
  adminSettings = _messages.MessageField('AdminSettings', 2)
  catalogIntegrationOptOut = _messages.BooleanField(3)
  classType = _messages.EnumField('ClassTypeValueValuesEnum', 4)
  consumerNetwork = _messages.StringField(5)
  controlledEgressConfig = _messages.MessageField('ControlledEgressConfig', 6)
  controlledEgressEnabled = _messages.BooleanField(7)
  createTime = _messages.StringField(8)
  customDomain = _messages.MessageField('CustomDomain', 9)
  denyMaintenancePeriod = _messages.MessageField('DenyMaintenancePeriod', 10)
  egressPublicIp = _messages.StringField(11)
  encryptionConfig = _messages.MessageField('EncryptionConfig', 12)
  fipsEnabled = _messages.BooleanField(13)
  geminiEnabled = _messages.BooleanField(14)
  ingressIpAllowlistConfig = _messages.MessageField('IngressIpAllowlistConfig', 15)
  ingressPrivateIp = _messages.StringField(16)
  ingressPublicIp = _messages.StringField(17)
  lastDenyMaintenancePeriod = _messages.MessageField('DenyMaintenancePeriod', 18)
  linkedLspProjectNumber = _messages.IntegerField(19)
  lookerUri = _messages.StringField(20)
  lookerVersion = _messages.StringField(21)
  maintenanceSchedule = _messages.MessageField('MaintenanceSchedule', 22)
  maintenanceWindow = _messages.MessageField('MaintenanceWindow', 23)
  name = _messages.StringField(24)
  oauthConfig = _messages.MessageField('OAuthConfig', 25)
  periodicExportConfig = _messages.MessageField('PeriodicExportConfig', 26)
  platformEdition = _messages.EnumField('PlatformEditionValueValuesEnum', 27)
  privateIpEnabled = _messages.BooleanField(28)
  pscConfig = _messages.MessageField('PscConfig', 29)
  pscEnabled = _messages.BooleanField(30)
  publicIpEnabled = _messages.BooleanField(31)
  releaseChannel = _messages.EnumField('ReleaseChannelValueValuesEnum', 32)
  reservedRange = _messages.StringField(33)
  satisfiesPzi = _messages.BooleanField(34)
  satisfiesPzs = _messages.BooleanField(35)
  softDeleteReason = _messages.EnumField('SoftDeleteReasonValueValuesEnum', 36)
  state = _messages.EnumField('StateValueValuesEnum', 37)
  suspendedTime = _messages.StringField(38)
  updateTime = _messages.StringField(39)
  userMetadata = _messages.MessageField('UserMetadata', 40)


class InstanceBackup(_messages.Message):
  r"""The details of a backup resource.

  Enums:
    StateValueValuesEnum: Output only. The current state of the backup.

  Fields:
    createTime: Output only. The time when the backup was started.
    encryptionConfig: Output only. Current status of the CMEK encryption
    expireTime: Output only. The time when the backup will be deleted.
    name: Immutable. The relative resource name of the backup, in the
      following form: `projects/{project_number}/locations/{location_id}/insta
      nces/{instance_id}/backups/{backup}`
    state: Output only. The current state of the backup.
  """

  class StateValueValuesEnum(_messages.Enum):
    r"""Output only. The current state of the backup.

    Values:
      STATE_UNSPECIFIED: The state of the backup is unknown.
      CREATING: The backup is being created.
      DELETING: The backup is being deleted.
      ACTIVE: The backup is active and ready to use.
      FAILED: The backup failed.
    """
    STATE_UNSPECIFIED = 0
    CREATING = 1
    DELETING = 2
    ACTIVE = 3
    FAILED = 4

  createTime = _messages.StringField(1)
  encryptionConfig = _messages.MessageField('EncryptionConfig', 2)
  expireTime = _messages.StringField(3)
  name = _messages.StringField(4)
  state = _messages.EnumField('StateValueValuesEnum', 5)


class ListInstanceBackupsResponse(_messages.Message):
  r"""Response from listing Looker instance backups.

  Fields:
    instanceBackups: The list of instances matching the request filters, up to
      the requested `page_size`.
    nextPageToken: If provided, a page token that can look up the next
      `page_size` results. If empty, the results list is exhausted.
    unreachable: Locations that could not be reached.
  """

  instanceBackups = _messages.MessageField('InstanceBackup', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListInstancesResponse(_messages.Message):
  r"""Response from ListInstances.

  Fields:
    instances: The list of instances matching the request filters, up to the
      requested ListInstancesRequest.pageSize.
    nextPageToken: If provided, a page token that can look up the next
      ListInstancesRequest.pageSize results. If empty, the results list is
      exhausted.
    unreachable: Locations that could not be reached.
  """

  instances = _messages.MessageField('Instance', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  unreachable = _messages.StringField(3, repeated=True)


class ListLocationsResponse(_messages.Message):
  r"""The response message for Locations.ListLocations.

  Fields:
    locations: A list of locations that matches the specified filter in the
      request.
    nextPageToken: The standard List next-page token.
  """

  locations = _messages.MessageField('Location', 1, repeated=True)
  nextPageToken = _messages.StringField(2)


class ListOperationsResponse(_messages.Message):
  r"""The response message for Operations.ListOperations.

  Fields:
    nextPageToken: The standard List next-page token.
    operations: A list of operations that matches the specified filter in the
      request.
    unreachable: Unordered list. Unreachable resources. Populated when the
      request sets `ListOperationsRequest.return_partial_success` and reads
      across collections. For example, when attempting to list all resources
      across all supported locations.
  """

  nextPageToken = _messages.StringField(1)
  operations = _messages.MessageField('Operation', 2, repeated=True)
  unreachable = _messages.StringField(3, repeated=True)


class Location(_messages.Message):
  r"""A resource that represents a Google Cloud location.

  Messages:
    LabelsValue: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    MetadataValue: Service-specific metadata. For example the available
      capacity at the given location.

  Fields:
    displayName: The friendly name for this location, typically a nearby city
      name. For example, "Tokyo".
    labels: Cross-service attributes for the location. For example
      {"cloud.googleapis.com/region": "us-east1"}
    locationId: The canonical id for this location. For example: `"us-east1"`.
    metadata: Service-specific metadata. For example the available capacity at
      the given location.
    name: Resource name for the location, which may vary between
      implementations. For example: `"projects/example-project/locations/us-
      east1"`
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class LabelsValue(_messages.Message):
    r"""Cross-service attributes for the location. For example
    {"cloud.googleapis.com/region": "us-east1"}

    Messages:
      AdditionalProperty: An additional property for a LabelsValue object.

    Fields:
      additionalProperties: Additional properties of type LabelsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a LabelsValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata. For example the available capacity at the
    given location.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  displayName = _messages.StringField(1)
  labels = _messages.MessageField('LabelsValue', 2)
  locationId = _messages.StringField(3)
  metadata = _messages.MessageField('MetadataValue', 4)
  name = _messages.StringField(5)


class LookerProjectsLocationsGetRequest(_messages.Message):
  r"""A LookerProjectsLocationsGetRequest object.

  Fields:
    name: Resource name for the location.
  """

  name = _messages.StringField(1, required=True)


class LookerProjectsLocationsInstancesBackupsCreateRequest(_messages.Message):
  r"""A LookerProjectsLocationsInstancesBackupsCreateRequest object.

  Fields:
    instanceBackup: A InstanceBackup resource to be passed as the request
      body.
    parent: Required. Format:
      projects/{project}/locations/{location}/instances/{instance}
  """

  instanceBackup = _messages.MessageField('InstanceBackup', 1)
  parent = _messages.StringField(2, required=True)


class LookerProjectsLocationsInstancesBackupsDeleteRequest(_messages.Message):
  r"""A LookerProjectsLocationsInstancesBackupsDeleteRequest object.

  Fields:
    name: Required. Format: projects/{project}/locations/{location}/instances/
      {instance}/backups/{backup}
  """

  name = _messages.StringField(1, required=True)


class LookerProjectsLocationsInstancesBackupsGetRequest(_messages.Message):
  r"""A LookerProjectsLocationsInstancesBackupsGetRequest object.

  Fields:
    name: Required. Format: `projects/{project}/locations/{location}/instances
      /{instance}/backups/{backup}`.
  """

  name = _messages.StringField(1, required=True)


class LookerProjectsLocationsInstancesBackupsListRequest(_messages.Message):
  r"""A LookerProjectsLocationsInstancesBackupsListRequest object.

  Fields:
    orderBy: Sort results. Default order is "create_time desc". Other
      supported fields are "state" and "expire_time".
      https://google.aip.dev/132#ordering
    pageSize: The maximum number of instances to return.
    pageToken: A page token received from a previous ListInstances request.
    parent: Required. Format:
      projects/{project}/locations/{location}/instances/{instance}.
  """

  orderBy = _messages.StringField(1)
  pageSize = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(3)
  parent = _messages.StringField(4, required=True)


class LookerProjectsLocationsInstancesCreateRequest(_messages.Message):
  r"""A LookerProjectsLocationsInstancesCreateRequest object.

  Fields:
    instance: A Instance resource to be passed as the request body.
    instanceId: Required. The unique instance identifier. Must contain only
      lowercase letters, numbers, or hyphens, with the first character a
      letter and the last a letter or a number. 63 characters maximum.
    parent: Required. Format: `projects/{project}/locations/{location}`.
  """

  instance = _messages.MessageField('Instance', 1)
  instanceId = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class LookerProjectsLocationsInstancesDeleteRequest(_messages.Message):
  r"""A LookerProjectsLocationsInstancesDeleteRequest object.

  Fields:
    force: Whether to force cascading delete.
    name: Required. Format:
      `projects/{project}/locations/{location}/instances/{instance}`.
  """

  force = _messages.BooleanField(1)
  name = _messages.StringField(2, required=True)


class LookerProjectsLocationsInstancesExportRequest(_messages.Message):
  r"""A LookerProjectsLocationsInstancesExportRequest object.

  Fields:
    exportInstanceRequest: A ExportInstanceRequest resource to be passed as
      the request body.
    name: Required. Format:
      `projects/{project}/locations/{location}/instances/{instance}`.
  """

  exportInstanceRequest = _messages.MessageField('ExportInstanceRequest', 1)
  name = _messages.StringField(2, required=True)


class LookerProjectsLocationsInstancesGetRequest(_messages.Message):
  r"""A LookerProjectsLocationsInstancesGetRequest object.

  Fields:
    name: Required. Format:
      `projects/{project}/locations/{location}/instances/{instance}`.
  """

  name = _messages.StringField(1, required=True)


class LookerProjectsLocationsInstancesImportRequest(_messages.Message):
  r"""A LookerProjectsLocationsInstancesImportRequest object.

  Fields:
    importInstanceRequest: A ImportInstanceRequest resource to be passed as
      the request body.
    name: Required. Format:
      `projects/{project}/locations/{location}/instances/{instance}`.
  """

  importInstanceRequest = _messages.MessageField('ImportInstanceRequest', 1)
  name = _messages.StringField(2, required=True)


class LookerProjectsLocationsInstancesListRequest(_messages.Message):
  r"""A LookerProjectsLocationsInstancesListRequest object.

  Fields:
    pageSize: The maximum number of instances to return. If unspecified at
      most 256 will be returned. The maximum possible value is 2048.
    pageToken: A page token received from a previous ListInstancesRequest.
    parent: Required. Format: `projects/{project}/locations/{location}`.
    showDeleted: Optional. Whether to include deleted instances in the
      response.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)
  showDeleted = _messages.BooleanField(4)


class LookerProjectsLocationsInstancesPatchRequest(_messages.Message):
  r"""A LookerProjectsLocationsInstancesPatchRequest object.

  Fields:
    instance: A Instance resource to be passed as the request body.
    name: Output only. Format:
      `projects/{project}/locations/{location}/instances/{instance}`.
    updateMask: Required. Field mask used to specify the fields to be
      overwritten in the Instance resource by the update. The fields specified
      in the mask are relative to the resource, not the full request. A field
      will be overwritten if it is in the mask.
  """

  instance = _messages.MessageField('Instance', 1)
  name = _messages.StringField(2, required=True)
  updateMask = _messages.StringField(3)


class LookerProjectsLocationsInstancesRestartRequest(_messages.Message):
  r"""A LookerProjectsLocationsInstancesRestartRequest object.

  Fields:
    name: Required. Format:
      `projects/{project}/locations/{location}/instances/{instance}`.
    restartInstanceRequest: A RestartInstanceRequest resource to be passed as
      the request body.
  """

  name = _messages.StringField(1, required=True)
  restartInstanceRequest = _messages.MessageField('RestartInstanceRequest', 2)


class LookerProjectsLocationsInstancesRestoreRequest(_messages.Message):
  r"""A LookerProjectsLocationsInstancesRestoreRequest object.

  Fields:
    name: Required. Instance being restored Format:
      projects/{project}/locations/{location}/instances/{instance}
    restoreInstanceRequest: A RestoreInstanceRequest resource to be passed as
      the request body.
  """

  name = _messages.StringField(1, required=True)
  restoreInstanceRequest = _messages.MessageField('RestoreInstanceRequest', 2)


class LookerProjectsLocationsInstancesUndeleteRequest(_messages.Message):
  r"""A LookerProjectsLocationsInstancesUndeleteRequest object.

  Fields:
    name: Required. Format:
      projects/{project}/locations/{location}/instances/{instance}
    undeleteInstanceRequest: A UndeleteInstanceRequest resource to be passed
      as the request body.
  """

  name = _messages.StringField(1, required=True)
  undeleteInstanceRequest = _messages.MessageField('UndeleteInstanceRequest', 2)


class LookerProjectsLocationsListRequest(_messages.Message):
  r"""A LookerProjectsLocationsListRequest object.

  Fields:
    extraLocationTypes: Optional. Do not use this field unless explicitly
      documented otherwise. This is primarily for internal usage.
    filter: A filter to narrow down results to a preferred subset. The
      filtering language accepts strings like `"displayName=tokyo"`, and is
      documented in more detail in [AIP-160](https://google.aip.dev/160).
    name: The resource that owns the locations collection, if applicable.
    pageSize: The maximum number of results to return. If not set, the service
      selects a default.
    pageToken: A page token received from the `next_page_token` field in the
      response. Send that page token to receive the subsequent page.
  """

  extraLocationTypes = _messages.StringField(1, repeated=True)
  filter = _messages.StringField(2)
  name = _messages.StringField(3, required=True)
  pageSize = _messages.IntegerField(4, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(5)


class LookerProjectsLocationsOperationsCancelRequest(_messages.Message):
  r"""A LookerProjectsLocationsOperationsCancelRequest object.

  Fields:
    cancelOperationRequest: A CancelOperationRequest resource to be passed as
      the request body.
    name: The name of the operation resource to be cancelled.
  """

  cancelOperationRequest = _messages.MessageField('CancelOperationRequest', 1)
  name = _messages.StringField(2, required=True)


class LookerProjectsLocationsOperationsDeleteRequest(_messages.Message):
  r"""A LookerProjectsLocationsOperationsDeleteRequest object.

  Fields:
    name: The name of the operation resource to be deleted.
  """

  name = _messages.StringField(1, required=True)


class LookerProjectsLocationsOperationsGetRequest(_messages.Message):
  r"""A LookerProjectsLocationsOperationsGetRequest object.

  Fields:
    name: The name of the operation resource.
  """

  name = _messages.StringField(1, required=True)


class LookerProjectsLocationsOperationsListRequest(_messages.Message):
  r"""A LookerProjectsLocationsOperationsListRequest object.

  Fields:
    filter: The standard list filter.
    name: The name of the operation's parent resource.
    pageSize: The standard list page size.
    pageToken: The standard list page token.
    returnPartialSuccess: When set to `true`, operations that are reachable
      are returned as normal, and those that are unreachable are returned in
      the ListOperationsResponse.unreachable field. This can only be `true`
      when reading across collections. For example, when `parent` is set to
      `"projects/example/locations/-"`. This field is not supported by default
      and will result in an `UNIMPLEMENTED` error if set unless explicitly
      documented otherwise in service or product specific documentation.
  """

  filter = _messages.StringField(1)
  name = _messages.StringField(2, required=True)
  pageSize = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(4)
  returnPartialSuccess = _messages.BooleanField(5)


class MaintenanceSchedule(_messages.Message):
  r"""Published upcoming future maintenance schedule.

  Fields:
    endTime: The scheduled end time for the maintenance.
    startTime: The scheduled start time for the maintenance.
  """

  endTime = _messages.StringField(1)
  startTime = _messages.StringField(2)


class MaintenanceWindow(_messages.Message):
  r"""Specifies the recurring maintenance window.

  Enums:
    DayOfWeekValueValuesEnum: Required. Day of the week for this
      MaintenanceWindow (in UTC).

  Fields:
    dayOfWeek: Required. Day of the week for this MaintenanceWindow (in UTC).
    startTime: Required. Time in UTC when the period starts. Maintenance will
      be scheduled within 60 minutes.
  """

  class DayOfWeekValueValuesEnum(_messages.Enum):
    r"""Required. Day of the week for this MaintenanceWindow (in UTC).

    Values:
      DAY_OF_WEEK_UNSPECIFIED: The day of the week is unspecified.
      MONDAY: Monday
      TUESDAY: Tuesday
      WEDNESDAY: Wednesday
      THURSDAY: Thursday
      FRIDAY: Friday
      SATURDAY: Saturday
      SUNDAY: Sunday
    """
    DAY_OF_WEEK_UNSPECIFIED = 0
    MONDAY = 1
    TUESDAY = 2
    WEDNESDAY = 3
    THURSDAY = 4
    FRIDAY = 5
    SATURDAY = 6
    SUNDAY = 7

  dayOfWeek = _messages.EnumField('DayOfWeekValueValuesEnum', 1)
  startTime = _messages.MessageField('TimeOfDay', 2)


class OAuthConfig(_messages.Message):
  r"""Looker instance OAuth login settings.

  Fields:
    clientId: Input only. Client ID from an external OAuth application. This
      is an input-only field, and thus will not be set in any responses.
    clientSecret: Input only. Client secret from an external OAuth
      application. This is an input-only field, and thus will not be set in
      any responses.
    sharedOauthClientEnabled: Optional. Whether to use the shared OAuth
      client. Instances specifying this field do not need to provide client_id
      and client_secret.
  """

  clientId = _messages.StringField(1)
  clientSecret = _messages.StringField(2)
  sharedOauthClientEnabled = _messages.BooleanField(3)


class Operation(_messages.Message):
  r"""This resource represents a long-running operation that is the result of
  a network API call.

  Messages:
    MetadataValue: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    ResponseValue: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

  Fields:
    done: If the value is `false`, it means the operation is still in
      progress. If `true`, the operation is completed, and either `error` or
      `response` is available.
    error: The error result of the operation in case of failure or
      cancellation.
    metadata: Service-specific metadata associated with the operation. It
      typically contains progress information and common metadata such as
      create time. Some services might not provide such metadata. Any method
      that returns a long-running operation should document the metadata type,
      if any.
    name: The server-assigned name, which is only unique within the same
      service that originally returns it. If you use the default HTTP mapping,
      the `name` should be a resource name ending with
      `operations/{unique_id}`.
    response: The normal, successful response of the operation. If the
      original method returns no data on success, such as `Delete`, the
      response is `google.protobuf.Empty`. If the original method is standard
      `Get`/`Create`/`Update`, the response should be the resource. For other
      methods, the response should have the type `XxxResponse`, where `Xxx` is
      the original method name. For example, if the original method name is
      `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MetadataValue(_messages.Message):
    r"""Service-specific metadata associated with the operation. It typically
    contains progress information and common metadata such as create time.
    Some services might not provide such metadata. Any method that returns a
    long-running operation should document the metadata type, if any.

    Messages:
      AdditionalProperty: An additional property for a MetadataValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MetadataValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ResponseValue(_messages.Message):
    r"""The normal, successful response of the operation. If the original
    method returns no data on success, such as `Delete`, the response is
    `google.protobuf.Empty`. If the original method is standard
    `Get`/`Create`/`Update`, the response should be the resource. For other
    methods, the response should have the type `XxxResponse`, where `Xxx` is
    the original method name. For example, if the original method name is
    `TakeSnapshot()`, the inferred response type is `TakeSnapshotResponse`.

    Messages:
      AdditionalProperty: An additional property for a ResponseValue object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ResponseValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  done = _messages.BooleanField(1)
  error = _messages.MessageField('Status', 2)
  metadata = _messages.MessageField('MetadataValue', 3)
  name = _messages.StringField(4)
  response = _messages.MessageField('ResponseValue', 5)


class OperationMetadata(_messages.Message):
  r"""Represents the metadata of the long-running operation.

  Fields:
    apiVersion: API version used to start the operation.
    createTime: The time the operation was created.
    endTime: The time the operation finished running.
    requestedCancellation: Identifies whether the user has requested
      cancellation of the operation. Operations that have successfully been
      cancelled have Operation.error value with a google.rpc.Status.code of 1,
      corresponding to `Code.CANCELLED`.
    statusMessage: Human-readable status of the operation, if any.
    target: Server-defined resource path for the target of the operation.
    verb: Name of the verb executed by the operation.
  """

  apiVersion = _messages.StringField(1)
  createTime = _messages.StringField(2)
  endTime = _messages.StringField(3)
  requestedCancellation = _messages.BooleanField(4)
  statusMessage = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class PeriodicExportConfig(_messages.Message):
  r"""Configuration for periodic export.

  Fields:
    gcsUri: Required. Cloud Storage bucket URI for periodic export. Format:
      gs://{bucket_name}
    kmsKey: Required. Name of the CMEK key in KMS. Format: projects/{project}/
      locations/{location}/keyRings/{key_ring}/cryptoKeys/{crypto_key}
    startTime: Required. Time in UTC to start the periodic export job.
  """

  gcsUri = _messages.StringField(1)
  kmsKey = _messages.StringField(2)
  startTime = _messages.MessageField('TimeOfDay', 3)


class PscConfig(_messages.Message):
  r"""Information for Private Service Connect (PSC) setup for a Looker
  instance.

  Fields:
    allowedVpcs: Optional. List of VPCs that are allowed ingress into looker.
      Format: projects/{project}/global/networks/{network}
    lookerServiceAttachmentUri: Output only. URI of the Looker service
      attachment.
    serviceAttachments: Optional. List of egress service attachment
      configurations.
  """

  allowedVpcs = _messages.StringField(1, repeated=True)
  lookerServiceAttachmentUri = _messages.StringField(2)
  serviceAttachments = _messages.MessageField('ServiceAttachment', 3, repeated=True)


class RestartInstanceRequest(_messages.Message):
  r"""Request options for restarting an instance."""


class RestoreInstanceRequest(_messages.Message):
  r"""Request options for restoring an instance

  Fields:
    backup: Required. Backup being used to restore the instance Format: projec
      ts/{project}/locations/{location}/instances/{instance}/backups/{backup}
  """

  backup = _messages.StringField(1)


class ServiceAttachment(_messages.Message):
  r"""Service attachment configuration.

  Enums:
    ConnectionStatusValueValuesEnum: Output only. Connection status.

  Fields:
    connectionStatus: Output only. Connection status.
    failureReason: Output only. Reason the service attachment creation failed.
      This value will only be populated if the service attachment encounters
      an issue during provisioning.
    localFqdn: Optional. Fully qualified domain name that will be used in the
      private DNS record created for the service attachment.
    localFqdns: Optional. List of fully qualified domain names that will be
      used in the private DNS record created for the service attachment.
    targetServiceAttachmentUri: Required. URI of the service attachment to
      connect to. Format: projects/{project}/regions/{region}/serviceAttachmen
      ts/{service_attachment}
  """

  class ConnectionStatusValueValuesEnum(_messages.Enum):
    r"""Output only. Connection status.

    Values:
      UNKNOWN: Connection status is unspecified.
      ACCEPTED: Connection is established and functioning normally.
      PENDING: Connection is not established (Looker tenant project hasn't
        been allowlisted).
      REJECTED: Connection is not established (Looker tenant project is
        explicitly in reject list).
      NEEDS_ATTENTION: Issue with target service attachment, e.g. NAT subnet
        is exhausted.
      CLOSED: Target service attachment does not exist. This status is a
        terminal state.
    """
    UNKNOWN = 0
    ACCEPTED = 1
    PENDING = 2
    REJECTED = 3
    NEEDS_ATTENTION = 4
    CLOSED = 5

  connectionStatus = _messages.EnumField('ConnectionStatusValueValuesEnum', 1)
  failureReason = _messages.StringField(2)
  localFqdn = _messages.StringField(3)
  localFqdns = _messages.StringField(4, repeated=True)
  targetServiceAttachmentUri = _messages.StringField(5)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class Status(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class TimeOfDay(_messages.Message):
  r"""Represents a time of day. The date and time zone are either not
  significant or are specified elsewhere. An API may choose to allow leap
  seconds. Related types are google.type.Date and `google.protobuf.Timestamp`.

  Fields:
    hours: Hours of a day in 24 hour format. Must be greater than or equal to
      0 and typically must be less than or equal to 23. An API may choose to
      allow the value "24:00:00" for scenarios like business closing time.
    minutes: Minutes of an hour. Must be greater than or equal to 0 and less
      than or equal to 59.
    nanos: Fractions of seconds, in nanoseconds. Must be greater than or equal
      to 0 and less than or equal to 999,999,999.
    seconds: Seconds of a minute. Must be greater than or equal to 0 and
      typically must be less than or equal to 59. An API may allow the value
      60 if it allows leap-seconds.
  """

  hours = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  minutes = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  nanos = _messages.IntegerField(3, variant=_messages.Variant.INT32)
  seconds = _messages.IntegerField(4, variant=_messages.Variant.INT32)


class UndeleteInstanceRequest(_messages.Message):
  r"""Request options for undeleting an instance."""


class UserMetadata(_messages.Message):
  r"""Metadata about users for a Looker instance.

  Fields:
    additionalDeveloperUserCount: Optional. The number of additional developer
      users the instance owner has purchased.
    additionalStandardUserCount: Optional. The number of additional standard
      users the instance owner has purchased.
    additionalViewerUserCount: Optional. The number of additional viewer users
      the instance owner has purchased.
  """

  additionalDeveloperUserCount = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  additionalStandardUserCount = _messages.IntegerField(2, variant=_messages.Variant.INT32)
  additionalViewerUserCount = _messages.IntegerField(3, variant=_messages.Variant.INT32)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The Calliope command help document markdown generator."""


import abc
import io
import re
import textwrap

from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import usage_text
from googlecloudsdk.core import properties
from googlecloudsdk.core.console import console_io
from googlecloudsdk.core.universe_descriptor import universe_descriptor
import six


_SPLIT = 78  # Split lines longer than this.
_SECTION_INDENT = 8  # Section or list within section indent.
_FIRST_INDENT = 2  # First line indent.
_SUBSEQUENT_INDENT = 6  # Subsequent line indent.
_SECOND_LINE_OFFSET = 2  # Used to create 2nd line indentation using markdown.
_GCLOUD_ROOT_SURFACES = frozenset([
    'gcloud',
    'gcloud alpha',
    'gcloud beta',
    'gcloud preview',
])


def _GetIndexFromCapsule(capsule):
  """Returns a help doc index line for a capsule line.

  The capsule line is a formal imperative sentence, preceded by optional
  (RELEASE-TRACK) or [TAG] tags, optionally with markdown attributes. The index
  line has no tags, is not capitalized and has no period, period.

  Args:
    capsule: The capsule line to convert to an index line.

  Returns:
    The help doc index line for a capsule line.
  """
  # Strip leading tags: <markdown>(TAG)<markdown> or <markdown>[TAG]<markdown>.
  capsule = re.sub(r'(\*?[\[(][A-Z]+[\])]\*? +)*', '', capsule)
  # Lower case first word if not an abbreviation.
  match = re.match(r'([A-Z])([^A-Z].*)', capsule)
  if match:
    capsule = match.group(1).lower() + match.group(2)
  # Strip trailing period.
  return capsule.rstrip('.')


class ExampleCommandLineSplitter(object):
  """Example command line splitter.

  Attributes:
    max_index: int, The max index to check in line.
    quote_char: str, The current quote char for quotes split across lines.
    quote_index: int, The index of quote_char in line or 0 if in previous line.
  """

  def __init__(self):
    self._max_index = _SPLIT - _SECTION_INDENT - _FIRST_INDENT
    self._quote_char = None
    self._quote_index = 0

  def _SplitInTwo(self, line):
    """Splits line into before and after, len(before) < self._max_index.

    Args:
      line: str, The line to split.

    Returns:
      (before, after)
        The line split into two parts. <before> is a list of strings that forms
        the first line of the split and <after> is a string containing the
        remainder of the line to split. The display width of <before> is
        < self._max_index. <before> contains the separator chars, including a
        newline.
    """
    punct_index = 0
    quoted_space_index = 0
    quoted_space_quote = None
    space_index = 0
    space_flag = False
    i = 0
    while i < self._max_index:
      c = line[i]
      i += 1
      if c == self._quote_char:
        self._quote_char = None
      elif self._quote_char:
        if c == ' ':
          quoted_space_index = i - 1
          quoted_space_quote = self._quote_char
      elif c in ('"', "'"):
        self._quote_char = c
        self._quote_index = i
        quoted_space_index = 0
      elif c == '\\':
        i += 1
      elif i < self._max_index:
        if c == ' ':
          # Split before a flag instead of the next arg; it could be the flag
          # value.
          if line[i] == '-':
            space_flag = True
            space_index = i
          elif space_flag:
            space_flag = False
          else:
            space_index = i
        elif c in (',', ';', '/', '|'):
          punct_index = i
        elif c == '=':
          space_flag = False
    separator = '\\\n'
    indent = _FIRST_INDENT
    if space_index:
      split_index = space_index
      indent = _SUBSEQUENT_INDENT
    elif quoted_space_index:
      split_index = quoted_space_index
      if quoted_space_quote == "'":
        separator = '\n'
      else:
        split_index += 1
    elif punct_index:
      split_index = punct_index
    else:
      split_index = self._max_index
    if split_index <= self._quote_index:
      self._quote_char = None
    else:
      self._quote_index = 0
    self._max_index = _SPLIT - _SECTION_INDENT - indent
    return [line[:split_index], separator, ' ' * indent], line[split_index:]

  def Split(self, line):
    """Splits a long example command line by inserting newlines.

    Args:
      line: str, The command line to split.

    Returns:
      str, The command line with newlines inserted.
    """
    lines = []
    while len(line) > self._max_index:
      before, line = self._SplitInTwo(line)
      lines.extend(before)
    lines.append(line)
    return ''.join(lines)


def NormalizeExampleSection(doc):
  """Removes line breaks and extra spaces in example commands.

  In command implementation, some example commands were manually broken into
  multiple lines with or without "\". This function removes these line
  breaks and let ExampleCommandLineSplitter to split the long commands
  centrally.

  This function will not change example commands in the following situations:

  1. If the command is in a code block, surrounded with ```sh...```.
  2. If the values are within a quote (single or double quote).

  Args:
    doc: str, help text to process.

  Returns:
    Modified help text.
  """
  example_sec_until_next_sec = re.compile(
      r'^## EXAMPLES\n(.+?)(\n+## )', flags=re.M | re.DOTALL
  )
  example_sec_until_end = re.compile(
      r'^## EXAMPLES\n(.+)', flags=re.M | re.DOTALL
  )
  match_example_sec = example_sec_until_next_sec.search(doc)
  match_example_sec_to_end = example_sec_until_end.search(doc)
  # no EXAMPLES section
  if not match_example_sec and not match_example_sec_to_end:
    return doc
  elif match_example_sec:
    selected_match = match_example_sec
  else:
    selected_match = match_example_sec_to_end
  doc_before_examples = doc[: selected_match.start(1)]
  example_section = doc[selected_match.start(1) : selected_match.end(1)]
  doc_after_example = doc[selected_match.end(1) :]

  pat_example_line = re.compile(r'^ *(\$ .*)$', re.M)
  pat_code_block = re.compile(r'^ *```sh(.+?```)', re.M | re.DOTALL)
  pos = 0
  res = ''
  while True:
    match_example_line = pat_example_line.search(example_section, pos)
    match_code_block = pat_code_block.search(example_section, pos)
    if not match_code_block and not match_example_line:
      break
    elif match_code_block and match_example_line:
      # If it found an example command line and a code block, pick the one
      # closer to the starting point.
      if match_code_block.start(1) > match_example_line.start(1):
        example, next_pos = UnifyExampleLine(
            example_section, match_example_line.start(1)
        )
        res += example_section[pos : match_example_line.start(1)] + example
        pos = next_pos
      else:
        res += example_section[pos : match_code_block.end(1)]
        pos = match_code_block.end(1)
    elif match_code_block:
      res += example_section[pos : match_code_block.end(1)]
      pos = match_code_block.end(1)
    else:
      example, next_pos = UnifyExampleLine(
          example_section, match_example_line.start(1)
      )
      res += example_section[pos : match_example_line.start(1)] + example
      pos = next_pos
  return doc_before_examples + (res + example_section[pos:]) + doc_after_example


def UnifyExampleLine(example_doc, pos):
  """Returns the example command line at pos in one single line.

  pos is the starting point of an example (starting with "$ ").
  This function removes "\n" and "\" and redundant spaces in the example line.
  The resulted example should be in one single line.

  Args:
    example_doc: str, Example section of the help text.
    pos: int, Position to start. pos will be the starting position of an example
      line.

  Returns:
    normalized example command, next starting position to search
  """
  pat_match_next_command = re.compile(
      r'\$\s+(.+?)(\n +\$\s+)', re.DOTALL
  )  # match consecutive commands.
  pat_match_empty_line_after_command = re.compile(
      r'\$\s+(.+?)(\n\s*\n|\n\+\n)', re.DOTALL
  )
  match_next_command = pat_match_next_command.match(example_doc, pos)
  match_empty_line_after_command = pat_match_empty_line_after_command.match(
      example_doc, pos
  )
  # reached to the end
  if not match_next_command and not match_empty_line_after_command:
    new_doc = example_doc.rstrip()
    pat = re.compile(r'\$\s+(.+)', re.DOTALL)
    match = pat.match(new_doc, pos)
    example = match.group(1)
    pat = re.compile(r'\\\n\s*')
    example = pat.sub('', example)  # remove extra \n and \ and spaces.
    example = RemoveSpacesLineBreaksFromExample(example)
    return '$ ' + example, len(new_doc)
  elif match_next_command and match_empty_line_after_command:
    if len(match_next_command.group(1)) > len(
        match_empty_line_after_command.group(1)
    ):
      selected_match = match_empty_line_after_command
    else:
      selected_match = match_next_command
  else:
    selected_match = (
        match_next_command
        if match_next_command
        else match_empty_line_after_command
    )
  example = selected_match.group(1)
  pat = re.compile(r'\\\n\s*')
  example = pat.sub('', example)
  example = RemoveSpacesLineBreaksFromExample(example)
  next_pos = selected_match.end(1)
  return '$ ' + example, next_pos


def _PrecedingBackslashCount(res):
  index = len(res) - 1
  while index >= 0 and res[index] == '\\':
    index -= 1
  return len(res) - index - 1


def RemoveSpacesLineBreaksFromExample(example):
  """Returns the example with redundant spaces and line breaks removed.

  If a character sequence is quoted (either single or double quote), we will
  not touch its value. Single quote is not allowed within single quote even
  with a preceding backslash. Double quote is allowed in double quote with
  preceding backslash though. If the spaces and line breaks are within quote,
  they are not touched.

  Args: example, str: Example line to process.
  """
  res = []
  example = example.strip()
  pos = 0
  while pos < len(example):
    c = example[pos]
    if c not in ['"', "'"]:  # outside quote
      if c == '\n':
        c = ' '  # remove line break
      if not (c == ' ' and res and res[-1] == ' '):  # remove redundant spaces
        res.append(c)
      pos += 1
    elif c == "'":  # see single quote
      res.append(c)
      pos += 1
      # proceed until seeing a closing single quote or exhausting example.
      while pos < len(example) and example[pos] != "'":
        res.append(example[pos])
        pos += 1
      if pos < len(example):  # see closing single quote
        res.append(example[pos])
        pos += 1
    else:  # see double quote
      res.append(example[pos])
      pos += 1
      # proceed until seeing a closing double quote or exhausting example.
      while pos < len(example) and not (
          example[pos] == '"' and _PrecedingBackslashCount(res) % 2 == 0
      ):
        res.append(example[pos])
        pos += 1
      if pos < len(example):  # see closing double quote
        res.append(example[pos])
        pos += 1

  return ''.join(res)


class MarkdownGenerator(six.with_metaclass(abc.ABCMeta, object)):
  """Command help markdown document generator base class.

  Attributes:
    _buf: Output document stream.
    _capsule: The one line description string.
    _command_name: The dotted command name.
    _command_path: The command path list.
    _doc: The output markdown document string.
    _docstring: The command docstring.
    _file_name: The command path name (used to name documents).
    _final_sections: The list of PrintFinalSections section names.
    _is_hidden: The command is hidden.
    _out: Output writer.
    _printed_sections: The set of already printed sections.
    _release_track: The calliope.base.ReleaseTrack.
  """

  def __init__(self, command_path, release_track, is_hidden):
    """Constructor.

    Args:
      command_path: The command path list.
      release_track: The base.ReleaseTrack of the command.
      is_hidden: The command is hidden if True.
    """
    self._command_path = command_path
    self._command_name = ' '.join(self._command_path)
    self._subcommands = None
    self._subgroups = None
    self._sort_top_level_args = None
    self._top = self._command_path[0] if self._command_path else ''
    self._buf = io.StringIO()
    self._out = self._buf.write
    self._capsule = ''
    self._docstring = ''
    self._final_sections = ['EXAMPLES', 'SEE ALSO']
    self._arg_sections = None
    self._sections = {}
    self._file_name = '_'.join(self._command_path)
    self._global_flags = set()
    self._is_hidden = is_hidden
    self._release_track = release_track
    self._printed_sections = set()

  @abc.abstractmethod
  def IsValidSubPath(self, sub_command_path):
    """Determines if the given sub command path is valid from this node.

    Args:
      sub_command_path: [str], The pieces of the command path.

    Returns:
      True, if the given path parts exist under this command or group node.
      False, if the sub path does not lead to a valid command or group.
    """
    pass

  @abc.abstractmethod
  def GetArguments(self):
    """Returns the command arguments."""
    pass

  def FormatExample(self, cmd, args, with_args):
    """Creates a link to the command reference from a command example.

    If with_args is False and the provided command includes args,
    returns None.

    Args:
      cmd: [str], a command.
      args: [str], args with the command.
      with_args: bool, whether the example is valid if it has args.

    Returns:
      (str) a representation of the command with a link to the reference, plus
      any args. | None, if the command isn't valid.
    """
    if args and not with_args:
      return None
    ref = '/'.join(cmd)
    command_link = 'link:' + ref + '[' + ' '.join(cmd) + ']'
    if args:
      command_link += ' ' + ' '.join(args)
    return command_link

  @property
  def is_root(self):
    """Determine if this node should be treated as a "root" of the CLI tree.

    The top element is the root, but we also treat any additional release tracks
    as a root so that global flags are shown there as well.

    Returns:
      True if this node should be treated as a root, False otherwise.
    """
    if len(self._command_path) == 1:
      return True
    elif len(self._command_path) == 2:
      tracks = [t.prefix for t in base.ReleaseTrack.AllValues()]
      if self._command_path[-1] in tracks:
        return True
    return False

  @property
  def is_group(self):
    """Returns True if this node is a command group."""
    return bool(self._subgroups or self._subcommands)

  @property
  def sort_top_level_args(self):
    """Returns whether to sort the top level arguments in markdown docs."""
    return self._sort_top_level_args

  @property
  def is_topic(self):
    """Returns True if this node is a topic command."""
    if (
        len(self._command_path) >= 3
        and self._command_path[1] == self._release_track.prefix
    ):
      command_index = 2
    else:
      command_index = 1
    return (
        len(self._command_path) >= (command_index + 1)
        and self._command_path[command_index] == 'topic'
    )

  def _ExpandHelpText(self, text):
    """Expand command {...} references in text.

    Args:
      text: The text chunk to expand.

    Returns:
      The expanded help text.
    """
    return console_io.LazyFormat(
        text or '',
        command=self._command_name,
        man_name=self._file_name,
        top_command=self._top,
        parent_command=' '.join(self._command_path[:-1]),
        grandparent_command=' '.join(self._command_path[:-2]),
        index=self._capsule,
        **self._sections,
    )

  def _SetArgSections(self):
    """Sets self._arg_sections in document order."""
    if self._arg_sections is None:
      self._arg_sections, self._global_flags = usage_text.GetArgSections(
          self.GetArguments(),
          self.is_root,
          self.is_group,
          self.sort_top_level_args,
      )

  def _SplitCommandFromArgs(self, cmd):
    """Splits cmd into command and args lists.

    The command list part is a valid command and the args list part is the
    trailing args.

    Args:
      cmd: [str], A command + args list.

    Returns:
      (command, args): The command and args lists.
    """
    # The bare top level command always works.
    if len(cmd) <= 1:
      return cmd, []
    # Skip the top level command name.
    skip = 1
    i = skip
    while i <= len(cmd):
      i += 1
      if not self.IsValidSubPath(cmd[skip:i]):
        i -= 1
        break
    return cmd[:i], cmd[i:]

  def _UserInput(self, msg):
    """Returns msg with user input markdown.

    Args:
      msg: str, The user input string.

    Returns:
      The msg string with embedded user input markdown.
    """
    return (
        base.MARKDOWN_CODE
        + base.MARKDOWN_ITALIC
        + msg
        + base.MARKDOWN_ITALIC
        + base.MARKDOWN_CODE
    )

  def _ArgTypeName(self, arg):
    """Returns the argument type name for arg."""
    return 'positional' if arg.is_positional else 'flag'

  def _IsGcloudSurfaceCommand(self):
    """Returns True if the command is the gcloud command."""
    return self._command_name in _GCLOUD_ROOT_SURFACES

  def PrintSectionHeader(self, name, sep=True):
    """Prints the section header markdown for name.

    Args:
      name: str, The manpage section name.
      sep: boolean, Add trailing newline.
    """
    self._printed_sections.add(name)
    self._out('\n\n## {name}\n'.format(name=name))
    if sep:
      self._out('\n')

  def PrintUniverseInformationSection(self, disable_header=False):
    """Prints the command line information section.

    The information section provides disclaimer information on whether a command
    is available in a particular universe domain.

    Args:
      disable_header: Disable printing the section header if True.
    """

    if properties.IsDefaultUniverse():
      return

    if not disable_header:
      self.PrintSectionHeader('INFORMATION')

    code = base.MARKDOWN_CODE
    em = base.MARKDOWN_ITALIC

    if self._command.IsUniverseCompatible() or self._IsGcloudSurfaceCommand():
      info_body = (
          f'{code}{self._command_name}{code} is supported in universe domain '
          f'{em}{properties.GetUniverseDomain()}{em}; however, some of the '
          'values used in the help text may not be available. Command examples '
          'may not work as-is and may requires changes before execution.'
      )
    else:
      info_body = (
          f'{code}{self._command_name}{code} is not available in '
          f'universe domain {em}{properties.GetUniverseDomain()}{em}.'
      )

    # print the informartion
    self._out(info_body)

    # print UNIVERSE ADDITIONAL INFO section
    self.PrintSectionIfExists('UNIVERSE ADDITIONAL INFO')

  def PrintNameSection(self, disable_header=False):
    """Prints the command line name section.

    Args:
      disable_header: Disable printing the section header if True.
    """
    if not disable_header:
      self.PrintSectionHeader('NAME')
    self._out(
        '{command} - {index}\n'.format(
            command=self._command_name,
            index=_GetIndexFromCapsule(self._capsule),
        )
    )

  def PrintSynopsisSection(self, disable_header=False):
    """Prints the command line synopsis section.

    Args:
      disable_header: Disable printing the section header if True.
    """
    if self.is_topic:
      return
    self._SetArgSections()
    # MARKDOWN_CODE is the default SYNOPSIS font style.
    code = base.MARKDOWN_CODE
    em = base.MARKDOWN_ITALIC
    if not disable_header:
      self.PrintSectionHeader('SYNOPSIS')
    self._out(
        '{code}{command}{code}'.format(code=code, command=self._command_name)
    )

    if self._subcommands and self._subgroups:
      self._out(' ' + em + 'GROUP' + em + ' | ' + em + 'COMMAND' + em)
    elif self._subcommands:
      self._out(' ' + em + 'COMMAND' + em)
    elif self._subgroups:
      self._out(' ' + em + 'GROUP' + em)

    # Generate the arg usage string with flags in section order.
    remainder_usage = []
    for section in self._arg_sections:
      self._out(' ')
      self._out(
          usage_text.GetArgUsage(
              section.args,
              markdown=True,
              top=True,
              remainder_usage=remainder_usage,
          )
      )
    if self._global_flags:
      self._out(' [' + em + self._top.upper() + '_WIDE_FLAG ...' + em + ']')
    if remainder_usage:
      self._out(' ')
      self._out(' '.join(remainder_usage))

    self._out('\n')

  def _PrintArgDefinition(self, arg, depth=0, single=False, in_group=False):
    """Prints a positional or flag arg definition list item at depth."""
    usage = usage_text.GetArgUsage(arg, definition=True, markdown=True)
    if not usage:
      return
    self._out(
        '\n{usage}{depth}\n'.format(
            usage=usage, depth=':' * (depth + _SECOND_LINE_OFFSET)
        )
    )
    if arg.is_required and (depth or in_group) and not single:
      modal = (
          '\n+\nThis {arg_type} argument must be specified if any of the other '
          'arguments in this group are specified.'
      ).format(arg_type=self._ArgTypeName(arg))
    else:
      modal = ''
    details = self.GetArgDetails(arg, depth=depth).replace('\n\n', '\n+\n')
    self._out('\n{details}{modal}\n'.format(details=details, modal=modal))

  def _PrintArgGroup(self, arg, depth=0, single=False, disable_header=False):
    """Prints an arg group definition list at depth."""
    args = (
        sorted(arg.arguments, key=usage_text.GetArgSortKey)
        if arg.sort_args
        else arg.arguments
    )
    heading = []
    if (arg.help or arg.is_mutex or arg.is_required) and not disable_header:
      if arg.help:
        heading.append(arg.help)
      if arg.disable_default_heading:
        pass
      elif len(args) == 1 or args[0].is_required:
        if arg.is_required:
          heading.append('This must be specified.')
      elif arg.is_mutex:
        if arg.is_required:
          heading.append('Exactly one of these must be specified:')
        else:
          heading.append('At most one of these can be specified:')
      elif arg.is_required:
        heading.append('At least one of these must be specified:')

    if not arg.is_hidden and heading:
      self._out(
          '\n{0} {1}\n\n'.format(
              ':' * (depth + _SECOND_LINE_OFFSET), '\n+\n'.join(heading)
          ).replace('\n\n', '\n+\n'),
      )
      if not (len(heading) == 1 and arg.help and heading[0] == arg.help):
        depth += 1
      heading = None

    has_non_group_arg = False
    for group_arg in args:
      if not group_arg.is_group and not group_arg.is_hidden:
        has_non_group_arg = True
        break

    # If the group has help, it is a real group and not just a wrapper.
    in_group = bool(arg.help)

    first_arg = True
    for a in args:
      if a.is_hidden:
        continue

      if a.is_group:
        single = False
        singleton = usage_text.GetSingleton(a)
        if singleton:
          if not a.help:
            a = singleton
          else:
            single = True
      if a.is_group and arg.is_mutex and has_non_group_arg:
        if not first_arg:
          intro = 'Or at least one of these can be specified:'
          if a.is_mutex:
            if a.is_required:
              intro = 'Or exactly one of these must be specified:'
            else:
              intro = 'Or at most one of these can be specified:'
          elif a.is_required:
            intro = 'Or at least one of these must be specified:'

          self._out(
              '\n{0} {1}\n\n'.format(
                  ':' * (depth + _SECOND_LINE_OFFSET),
                  '\n+\n'.join([intro]),
              ).replace('\n\n', '\n+\n'),
          )

          next_depth = depth + 1
          if a.help:
            self._out(
                '\n{0} {1}\n\n'.format(
                    ':' * (next_depth + _SECOND_LINE_OFFSET),
                    '\n+\n'.join([a.help]),
                ).replace('\n\n', '\n+\n'),
            )

          self._PrintArgGroup(
              a, depth=next_depth, single=single, disable_header=True
          )
        else:
          self._PrintArgGroup(a, depth=depth + 1, single=single)
      elif a.is_group:
        self._PrintArgGroup(a, depth=depth, single=single)
      else:
        self._PrintArgDefinition(
            a, depth=depth, single=single, in_group=in_group
        )
      first_arg = False

  def PrintPositionalDefinition(self, arg, depth=0):
    self._out(
        '\n{usage}{depth}\n'.format(
            usage=usage_text.GetPositionalUsage(arg, markdown=True),
            depth=':' * (depth + _SECOND_LINE_OFFSET),
        )
    )
    self._out('\n{arghelp}\n'.format(arghelp=self.GetArgDetails(arg)))

  def PrintFlagDefinition(self, flag, disable_header=False, depth=0):
    """Prints a flags definition list item.

    Args:
      flag: The flag object to display.
      disable_header: Disable printing the section header if True.
      depth: The indentation depth at which to print arg help text.
    """
    if not disable_header:
      self._out('\n')
    self._out(
        '{usage}{depth}\n'.format(
            usage=usage_text.GetFlagUsage(flag, markdown=True),
            depth=':' * (depth + _SECOND_LINE_OFFSET),
        )
    )
    self._out('\n{arghelp}\n'.format(arghelp=self.GetArgDetails(flag)))

  def PrintFlagSection(self, heading, arg, disable_header=False):
    """Prints a flag section.

    Args:
      heading: The flag section heading name.
      arg: The flag args / group.
      disable_header: Disable printing the section header if True.
    """
    if not disable_header:
      self.PrintSectionHeader(heading, sep=False)
    self._PrintArgGroup(arg)

  def PrintPositionalsAndFlagsSections(self, disable_header=False):
    """Prints the positionals and flags sections.

    Args:
      disable_header: Disable printing the section header if True.
    """
    if self.is_topic:
      return
    self._SetArgSections()

    # List the sections in order.
    for section in self._arg_sections:
      self.PrintFlagSection(
          section.heading, section.args, disable_header=disable_header
      )

    if self._global_flags:
      if not disable_header:
        self.PrintSectionHeader(
            '{} WIDE FLAGS'.format(self._top.upper()), sep=False
        )
      # NOTE: We need two newlines before 'Run' for a paragraph break.
      self._out(
          '\nThese flags are available to all commands: {}.'
          '\n\nRun *$ {} help* for details.\n'.format(
              ', '.join(sorted(self._global_flags)), self._top
          )
      )

  def PrintSubGroups(self, disable_header=False):
    """Prints the subgroup section if there are subgroups.

    Args:
      disable_header: Disable printing the section header if True.
    """
    if self._subgroups:
      self.PrintCommandSection(
          'GROUP', self._subgroups, disable_header=disable_header
      )

  def PrintSubCommands(self, disable_header=False):
    """Prints the subcommand section if there are subcommands.

    Args:
      disable_header: Disable printing the section header if True.
    """
    if self._subcommands:
      if self.is_topic:
        self.PrintCommandSection(
            'TOPIC',
            self._subcommands,
            is_topic=True,
            disable_header=disable_header,
        )
      else:
        self.PrintCommandSection(
            'COMMAND', self._subcommands, disable_header=disable_header
        )

  def PrintSectionIfExists(self, name, default=None, disable_header=False):
    """Print a section name if it exists.

    Args:
      name: str, The manpage section name.
      default: str, Default help_stuff if section name is not defined.
      disable_header: Disable printing the section header if True.
    """
    if name in self._printed_sections:
      return
    help_stuff = self._sections.get(name, default)
    if not help_stuff:
      return
    if callable(help_stuff):
      help_message = help_stuff()
    else:
      help_message = help_stuff
    if not disable_header:
      self.PrintSectionHeader(name)
    self._out(
        '{message}\n'.format(message=textwrap.dedent(help_message).strip())
    )

  def PrintExtraSections(self, disable_header=False):
    """Print extra sections not in excluded_sections.

    Extra sections are sections that have not been printed yet.
    PrintSectionIfExists() skips sections that have already been printed.

    Args:
      disable_header: Disable printing the section header if True.
    """
    excluded_sections = set(
        self._final_sections + ['NOTES', 'UNIVERSE ADDITIONAL INFO']
    )
    for section in sorted(self._sections):
      if section.isupper() and section not in excluded_sections:
        self.PrintSectionIfExists(section, disable_header=disable_header)

  def PrintFinalSections(self, disable_header=False):
    """Print the final sections in order.

    Args:
      disable_header: Disable printing the section header if True.
    """
    for section in self._final_sections:
      self.PrintSectionIfExists(section, disable_header=disable_header)
    self.PrintNotesSection(disable_header=disable_header)

  def PrintCommandSection(
      self, name, subcommands, is_topic=False, disable_header=False
  ):
    """Prints a group or command section.

    Args:
      name: str, The section name singular form.
      subcommands: dict, The subcommand dict.
      is_topic: bool, True if this is a TOPIC subsection.
      disable_header: Disable printing the section header if True.
    """
    # Determine if the section has any content.
    content = ''
    for subcommand, help_info in sorted(six.iteritems(subcommands)):
      if self._is_hidden or not help_info.is_hidden:
        # If this group is already hidden, we can safely include hidden
        # sub-items.  Else, only include them if they are not hidden.
        content += '\n*link:{ref}[{cmd}]*::\n\n{txt}\n'.format(
            ref='/'.join(self._command_path + [subcommand]),
            cmd=subcommand,
            txt=help_info.help_text,
        )
    if content:
      if not disable_header:
        self.PrintSectionHeader(name + 'S')
      if is_topic:
        self._out('The supplementary help topics are:\n')
      else:
        self._out(
            '{cmd} is one of the following:\n'.format(cmd=self._UserInput(name))
        )
      self._out(content)

  def GetNotes(self):
    """Returns the explicit NOTES section contents."""
    return self._sections.get('NOTES')

  def PrintNotesSection(self, disable_header=False):
    """Prints the NOTES section if needed.

    Args:
      disable_header: Disable printing the section header if True.
    """
    notes = self.GetNotes()
    if notes:
      if not disable_header:
        self.PrintSectionHeader('NOTES')
      if notes:
        self._out(notes + '\n\n')

  def GetArgDetails(self, arg, depth=0):
    """Returns the detailed help message for the given arg."""
    if getattr(arg, 'detailed_help', None):
      raise ValueError(
          '{}: Use add_argument(help=...) instead of detailed_help="""{}""".'
          .format(self._command_name, getattr(arg, 'detailed_help'))
      )
    return usage_text.GetArgDetails(arg, depth=depth)

  def _ExpandFormatReferences(self, doc):
    """Expand {...} references in doc."""
    doc = self._ExpandHelpText(doc)
    doc = NormalizeExampleSection(doc)

    # Split long $ ... example lines.
    pat = re.compile(
        r'^ *(\$ .{%d,})$' % (_SPLIT - _FIRST_INDENT - _SECTION_INDENT), re.M
    )
    pos = 0
    rep = ''
    while True:
      match = pat.search(doc, pos)
      if not match:
        break
      rep += doc[pos : match.start(1)] + ExampleCommandLineSplitter().Split(
          doc[match.start(1) : match.end(1)]
      )
      pos = match.end(1)
    if rep:
      doc = rep + doc[pos:]
    return doc

  def _IsNotThisCommand(self, cmd):
    # We should not include the link if it refers to the current page, per
    # our research with screen readers. (See b/1723464.)
    return '.'.join(cmd) != '.'.join(self._command_path)

  def _LinkMarkdown(self, doc, pat, with_args=True):
    """Build a representation of a doc, finding all command examples.

    Finds examples of both inline commands and commands on their own line.

    Args:
      doc: str, the doc to find examples in.
      pat: the compiled regexp pattern to match against (the "command" match
        group).
      with_args: bool, whether the examples are valid if they also have args.

    Returns:
      (str) The final representation of the doc.
    """
    pos = 0
    rep = ''
    while True:
      match = pat.search(doc, pos)
      if not match:
        break
      cmd, args = self._SplitCommandFromArgs(match.group('command').split(' '))
      lnk = self.FormatExample(cmd, args, with_args=with_args)
      if self._IsNotThisCommand(cmd) and lnk:
        rep += doc[pos : match.start('command')] + lnk
      else:
        # Skip invalid commands.
        rep += doc[pos : match.end('command')]
      rep += doc[match.end('command') : match.end('end')]
      pos = match.end('end')
    if rep:
      doc = rep + doc[pos:]
    return doc

  def InlineCommandExamplePattern(self):
    """Regex to search for inline command examples enclosed in ` or *.

    Contains a 'command' group and an 'end' group which will be used
    by the regexp search later.

    Returns:
      (str) the regex pattern, including a format string for the 'top'
      command.
    """
    # This pattern matches "([`*]){top} {arg}*\1" where {top}...{arg} is a
    # known command. The negative lookbehind prefix prevents hyperlinks in
    # SYNOPSIS sections and as the first line in a paragraph.
    return (
        r'(?<!\n\n)(?<!\*\(ALPHA\)\* )(?<!\*\(BETA\)\* )(?<!\*\(PREVIEW\)\* )'
        r'([`*])(?P<command>{top}( [a-z][-a-z0-9]*)*)(?P<end>\1)'.format(
            top=re.escape(self._top)
        )
    )

  def _AddCommandLinkMarkdown(self, doc):
    r"""Add ([`*])command ...\1 link markdown to doc."""
    if not self._command_path:
      return doc
    pat = re.compile(self.InlineCommandExamplePattern())
    doc = self._LinkMarkdown(doc, pat, with_args=False)
    return doc

  def CommandLineExamplePattern(self):
    """Regex to search for command examples starting with '$ '.

    Contains a 'command' group and an 'end' group which will be used
    by the regexp search later.

    Returns:
      (str) the regex pattern, including a format string for the 'top'
      command.
    """
    # This pattern matches "$ {top} {arg}*" where each arg is lower case and
    # does not start with example-, my-, or sample-. This follows the style
    # guide rule that user-supplied args to example commands contain upper case
    # chars or start with example-, my-, or sample-. The trailing .? allows for
    # an optional punctuation character before end of line. This handles cases
    # like ``... run $ <top> foo bar.'' at the end of a sentence.
    # The <end> group ends at the same place as the command group, without
    # the punctuation or newlines.
    return (
        r'\$ (?P<end>(?P<command>{top}((?: (?!(example|my|sample)-)'
        r'[a-z][-a-z0-9]*)*))).?[ `\n]'.format(top=re.escape(self._top))
    )

  def _AddCommandLineLinkMarkdown(self, doc):
    """Add $ command ... link markdown to doc."""
    if not self._command_path:
      return doc
    pat = re.compile(self.CommandLineExamplePattern())
    doc = self._LinkMarkdown(doc, pat, with_args=True)
    return doc

  def _AddManPageLinkMarkdown(self, doc):
    """Add <top> ...(1) man page link markdown to doc."""
    if not self._command_path:
      return doc
    pat = re.compile(r'(\*?(' + self._top + r'(?:[-_ a-z])*)\*?)\(1\)')
    pos = 0
    rep = ''
    while True:
      match = pat.search(doc, pos)
      if not match:
        break
      cmd = match.group(2).replace('_', ' ')
      ref = cmd.replace(' ', '/')
      lnk = '*link:' + ref + '[' + cmd + ']*'
      rep += doc[pos : match.start(2)] + lnk
      pos = match.end(1)
    if rep:
      doc = rep + doc[pos:]
    return doc

  def _FixAirQuotesMarkdown(self, doc):
    """Change ``.*[[:alnum:]]{2,}.*'' quotes => _UserInput(*) in doc."""

    # Double ``air quotes'' on strings with no identifier chars or groups of
    # singleton identifier chars are literal. All other double air quote forms
    # are converted to unquoted strings with the _UserInput() font
    # embellishment. This is a subjective choice for aesthetically pleasing
    # renderings.
    pat = re.compile(r"[^`](``([^`']*)'')")
    pos = 0
    rep = ''
    for match in pat.finditer(doc):
      if re.search(r'\w\w', match.group(2)):
        quoted_string = self._UserInput(match.group(2))
      else:
        quoted_string = match.group(1)
      rep += doc[pos : match.start(1)] + quoted_string
      pos = match.end(1)
    if rep:
      doc = rep + doc[pos:]
    return doc

  def _IsUniverseCompatible(self):
    return (
        not properties.IsDefaultUniverse()
        and not isinstance(self._command, dict)
        and self._command.IsUniverseCompatible()
    )

  def _ReplaceGDULinksWithUniverseLinks(self, doc):
    """Replace static GDU Links with Universe Links."""

    # Replace links only for other universes and
    # command is available in the universe.
    if self._IsUniverseCompatible():
      doc = re.sub(
          r'cloud.google.com',
          universe_descriptor.GetUniverseDocumentDomain(),
          doc,
      )

    return doc

  def Edit(self, doc=None):
    """Applies edits to a copy of the generated markdown in doc.

    The sub-edit method call order might be significant. This method allows
    the combined edits to be tested without relying on the order.

    Args:
      doc: The markdown document string to edit, None for the output buffer.

    Returns:
      An edited copy of the generated markdown.
    """
    if doc is None:
      doc = self._buf.getvalue()
    doc = self._ExpandFormatReferences(doc)
    doc = self._AddCommandLineLinkMarkdown(doc)
    doc = self._AddCommandLinkMarkdown(doc)
    doc = self._AddManPageLinkMarkdown(doc)
    doc = self._FixAirQuotesMarkdown(doc)
    doc = self._ReplaceGDULinksWithUniverseLinks(doc)
    return doc

  def Generate(self):
    """Generates markdown for the command, group or topic, into a string.

    Returns:
      An edited copy of the generated markdown.
    """
    self._out('# {0}(1)\n'.format(self._file_name.upper()))
    # Disclaimer info will be printed only for other universes
    self.PrintUniverseInformationSection()
    self.PrintNameSection()
    self.PrintSynopsisSection()
    self.PrintSectionIfExists('DESCRIPTION')
    self.PrintSectionIfExists('EXAMPLES')
    self.PrintPositionalsAndFlagsSections()
    self.PrintSubGroups()
    self.PrintSubCommands()
    self.PrintExtraSections()
    self.PrintFinalSections()
    return self.Edit()


class CommandMarkdownGenerator(MarkdownGenerator):
  """Command help markdown document generator.

  Attributes:
    _command: The CommandCommon instance for command.
    _root_command: The root CLI command instance.
    _subcommands: The dict of subcommand help indexed by subcommand name.
    _subgroups: The dict of subgroup help indexed by subcommand name.
  """

  def __init__(self, command):
    """Constructor.

    Args:
      command: A calliope._CommandCommon instance. Help is extracted from this
        calliope command, group or topic.
    """
    self._command = command
    command.LoadAllSubElements()
    # pylint: disable=protected-access
    self._root_command = command._TopCLIElement()
    super(CommandMarkdownGenerator, self).__init__(
        command.GetPath(), command.ReleaseTrack(), command.IsHidden()
    )
    self._capsule = self._command.short_help
    self._docstring = self._command.long_help
    self._ExtractSectionsFromDocstring(self._docstring)
    self._sections['description'] = self._sections.get('DESCRIPTION', '')
    self._sections.update(getattr(self._command, 'detailed_help', {}))

    # Guidance sidecar help augmentation.
    try:
      guidance = getattr(self._command._common_type, 'guidance', None)
      if guidance:
        if isinstance(guidance, dict):
          self._sections['GUIDANCE'] = guidance.get('content')
    except Exception:  # pylint: disable=broad-except
      # Fail gracefully if we cannot resolve the guidance.
      pass

    self._subcommands = command.GetSubCommandHelps()
    self._subgroups = command.GetSubGroupHelps()
    self._sort_top_level_args = command.ai.sort_args

  def _SetSectionHelp(self, name, lines):
    """Sets section name help composed of lines.

    Args:
      name: The section name.
      lines: The list of lines in the section.
    """
    # Strip leading empty lines.
    while lines and not lines[0]:
      lines = lines[1:]
    # Strip trailing empty lines.
    while lines and not lines[-1]:
      lines = lines[:-1]
    if lines:
      self._sections[name] = '\n'.join(lines)

  def _ExtractSectionsFromDocstring(self, docstring):
    """Extracts section help from the command docstring."""
    name = 'DESCRIPTION'
    lines = []
    for line in textwrap.dedent(docstring).strip().splitlines():
      # '## \n' is not section markdown.
      if len(line) >= 4 and line.startswith('## '):
        self._SetSectionHelp(name, lines)
        name = line[3:]
        lines = []
      else:
        lines.append(line)
    self._SetSectionHelp(name, lines)

  def IsValidSubPath(self, sub_command_path):
    """Returns True if the given sub command path is valid from this node."""
    return self._root_command.IsValidSubPath(sub_command_path)

  def GetArguments(self):
    """Returns the command arguments."""
    return self._command.ai.arguments

  def GetNotes(self):
    """Returns the explicit and auto-generated NOTES section contents."""
    return self._command.GetNotesHelpSection(self._sections.get('NOTES'))


def Markdown(command):
  """Generates and returns the help markdown document for command.

  Args:
    command: The CommandCommon command instance.

  Returns:
    The markdown document string.
  """
  return CommandMarkdownGenerator(command).Generate()



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Executes a statement on a Cloud SQL instance."""

from googlecloudsdk.api_lib.sql import api_util
from googlecloudsdk.api_lib.sql import validate
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.sql import flags
from googlecloudsdk.core import properties
from googlecloudsdk.core.util import files

DESCRIPTION = """\
    Executes a statement on a Cloud SQL instance. It will use the
    credentials of the specified Google Cloud account to connect to the
    instance, so an IAM user with the same name must exist in the instance.
    WARNING: The requests and responses might transit through intermediate
    locations between your client and the location of the target instance.
    """

EXAMPLES = """\
    To execute a statement on a Cloud SQL instance, run:

    $ {command} my-instance-name --sql="ALTER TABLE employees RENAME TO personnel;" --database=my-db
    """

DETAILED_HELP = {
    'DESCRIPTION': DESCRIPTION,
    'EXAMPLES': EXAMPLES,
}


@base.ReleaseTracks(
    base.ReleaseTrack.GA, base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA
)
@base.DefaultUniverseOnly
class ExecuteSql(base.Command):
  """Executes a statement on a Cloud SQL instance."""

  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go on
        the command line after this command. Positional arguments are allowed.
    """
    parser.add_argument(
        'instance',
        completer=flags.InstanceCompleter,
        help='Cloud SQL instance ID.',
    )
    parser.add_argument(
        '--sql',
        required=True,
        help=(
            'SQL statement(s) to execute. It supports multiple statements as'
            " well. When it starts with the character '@', the rest should be a"
            ' filepath to read the SQL statement(s) from. For example,'
            ' --sql=@my_script.sql.'
        ),
    )
    parser.add_argument(
        '--row_limit',
        type=int,
        required=False,
        help='Maximum number of rows to return. The default is unlimited.',
    )
    parser.add_argument(
        '--partial_result_mode',
        choices={
            'PARTIAL_RESULT_MODE_UNSPECIFIED': (
                'Unspecified mode, effectively the same as'
                ' <code>FAIL_PARTIAL_RESULT</code>.'
            ),
            'FAIL_PARTIAL_RESULT': (
                'If the complete result is unavailable, returns an error'
                " and doesn't return the partial result."
            ),
            'ALLOW_PARTIAL_RESULT': (
                'If the complete result is unavailable, returns a partial'
                ' result, marks the field <code>partial_result</code> to'
                " <code>true</code>, and doesn't throw an error."
            ),
        },
        required=False,
        default=None,
        help=(
            'Controls how the API responds when the SQL execution result'
            ' is incomplete due to size limit or other reasons. The default'
            ' mode is to return an error instead of returning a partial result.'
        ),
    )
    flags.AddDatabase(
        parser,
        'Database on which the statement is executed.',
    )
    parser.add_argument(
        '--password-secret-version',
        required=False,
        help=(
            'The resource name of the Secret Manager secret holding the'
            ' password for the database user. The secret should be created'
            ' using the regional endpoint and stored in the same region as'
            ' the Cloud SQL instance. The expected resource name format is'
            ' <code>projects/{project}/locations/{location}/secrets/{secret}/versions/{secret_version}</code>.'
            ' Used together with <code>--user</code>.'
        ),
    )
    parser.add_argument(
        '--user',
        required=False,
        help=(
            'The database user to authenticate as. Used together with'
            ' <code>--password-secret-version</code>.'
        ),
    )

  def Run(self, args):
    """Executes a statement on a Cloud SQL instance.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      A dict object representing the execution result.
    """
    client = api_util.SqlClient(api_util.API_VERSION_DEFAULT)
    sql_client = client.sql_client
    sql_messages = client.sql_messages

    if args.password_secret_version and not args.user:
      raise exceptions.RequiredArgumentException(
          '--user',
          '`--user` is required when `--password-secret-version` is specified.',
      )

    if args.user and not args.password_secret_version:
      raise exceptions.RequiredArgumentException(
          '--password-secret-version',
          '`--password-secret-version` is required when `--user` is specified.',
      )

    validate.ValidateInstanceName(args.instance)
    instance_ref = client.resource_parser.Parse(
        args.instance,
        params={'project': properties.VALUES.core.project.GetOrFail},
        collection='sql.instances',
    )
    if args.sql.startswith('@'):
      sql = files.ReadFileContents(args.sql[1:])
    else:
      sql = args.sql
    if args.partial_result_mode:
      partial_result_mode = sql_messages.ExecuteSqlPayload.PartialResultModeValueValuesEnum.lookup_by_name(
          args.partial_result_mode
      )
    else:
      partial_result_mode = None
    if args.password_secret_version:
      # Password authentication via Secret Manager
      req_body = sql_messages.ExecuteSqlPayload(
          sqlStatement=sql,
          database=args.database,
          rowLimit=args.row_limit,
          partialResultMode=partial_result_mode,
          user=args.user,
          passwordSecretVersion=args.password_secret_version,
          application='gcloud',
      )
    else:
      # IAM authentication
      req_body = sql_messages.ExecuteSqlPayload(
          sqlStatement=sql,
          database=args.database,
          rowLimit=args.row_limit,
          partialResultMode=partial_result_mode,
          autoIamAuthn=True,
          application='gcloud',
      )
    return sql_client.instances.ExecuteSql(
        sql_messages.SqlInstancesExecuteSqlRequest(
            instance=instance_ref.instance,
            project=instance_ref.project,
            executeSqlPayload=req_body,
        ),
    )
